@@ -77,6 +77,8 @@
 #include "g_game.h"
 #include "v_trans.h" // [crispy] colored "always run" message
 
+#include "deh_main.h" // [crispy] for demo footer
+#include "memio.h"
 
 #define SAVEGAMESIZE	0x2c000
 
@@ -1274,7 +1276,6 @@ void G_Ticker (void)
              && turbodetected[i])
             {
                 static char turbomessage[80];
-                extern char *player_names[4];
                 M_snprintf(turbomessage, sizeof(turbomessage),
                            "%s is turbo!", player_names[i]);
                 players[consoleplayer].message = turbomessage;
@@ -1726,7 +1727,6 @@ static const int npars[9] =
 // G_DoCompleted 
 //
 boolean		secretexit; 
-extern char*	pagename; 
  
 void G_ExitLevel (void) 
 { 
@@ -2160,8 +2160,6 @@ void G_DoWorldDone (void)
 // G_InitFromSavegame
 // Can be called by the startup code or the menu task. 
 //
-extern boolean setsizeneeded;
-void R_ExecuteSetViewSize (void);
 
 
 void G_LoadGame (char* name) 
@@ -3132,6 +3130,77 @@ void G_TimeDemo (char* name)
     gameaction = ga_playdemo; 
 } 
  
+#define DEMO_FOOTER_SEPARATOR "\n"
+
+static void G_AddDemoFooter(void)
+{
+    int i;
+    size_t len = 0;
+    char *tmp, **filenames;
+
+    MEMFILE *stream = mem_fopen_write();
+
+    filenames = W_GetWADFileNames();
+
+    if (!filenames)
+    {
+        return;
+    }
+
+    tmp = M_StringJoin(PACKAGE_STRING, DEMO_FOOTER_SEPARATOR,
+            "-iwad \"", M_BaseName(filenames[0]), "\"", NULL);
+    mem_fputs(tmp, stream);
+    free(tmp);
+
+    if (filenames[1])
+    {
+        mem_fputs(" -file", stream);
+
+        for (i = 1; filenames[i]; i++)
+        {
+            tmp = M_StringJoin(" \"", M_BaseName(filenames[i]), "\"", NULL);
+            mem_fputs(tmp, stream);
+            free(tmp);
+        }
+    }
+
+    filenames = DEH_GetFileNames();
+
+    if (filenames)
+    {
+        mem_fputs(" -deh", stream);
+
+        for (i = 0; filenames[i]; i++)
+        {
+            tmp = M_StringJoin(" \"", M_BaseName(filenames[i]), "\"", NULL);
+            mem_fputs(tmp, stream);
+            free(tmp);
+        }
+    }
+
+    tmp = M_StringJoin(" -gameversion ", D_GetGameVersionCmd(), NULL);
+    mem_fputs(tmp, stream);
+    free(tmp);
+
+    if (M_CheckParm("-solo-net"))
+    {
+        mem_fputs(" -solo-net", stream);
+    }
+
+    mem_fputs(DEMO_FOOTER_SEPARATOR, stream);
+
+    mem_get_buf(stream, (void **)&tmp, &len);
+
+    while (demo_p > demoend - len)
+    {
+        IncreaseDemoBuffer();
+    }
+
+    memcpy(demo_p, tmp, len);
+    demo_p += len;
+
+    mem_fclose(stream);
+}
  
 /* 
 =================== 
@@ -3220,18 +3289,23 @@ boolean G_CheckDemoStatus (void)
  
     if (demorecording) 
     { 
+	boolean success;
+	char *msg;
+
 	*demo_p++ = DEMOMARKER; 
-	M_WriteFile (demoname, demobuffer, demo_p - demobuffer); 
+	G_AddDemoFooter();
+	success = M_WriteFile (demoname, demobuffer, demo_p - demobuffer);
+	msg = success ? "Demo %s recorded%c" : "Failed to record Demo %s%c";
 	Z_Free (demobuffer); 
 	demorecording = false; 
 	// [crispy] if a new game is started during demo recording, start a new demo
 	if (gameaction != ga_newgame)
 	{
-	I_Error ("Demo %s recorded",demoname); 
+	    I_Error (msg, demoname, '\0');
 	}
 	else
 	{
-	    fprintf(stderr, "Demo %s recorded\n",demoname);
+	    fprintf(stderr, msg, demoname, '\n');
 	}
     } 
 	 
