@@ -235,6 +235,40 @@ static void P_ReadOldSpecial (const char *key)
 	}
 }
 
+// sector->rlightlevel
+
+static void P_WriteRLightlevel (const char *key)
+{
+	int i;
+	sector_t *sector;
+
+	for (i = 0, sector = sectors; i < numsectors; i++, sector++)
+	{
+		if (sector->rlightlevel != sector->lightlevel)
+		{
+			M_snprintf(line, MAX_LINE_LEN, "%s %d %d\n",
+			           key,
+			           i,
+			           (int)sector->rlightlevel);
+			fputs(line, save_stream);
+		}
+	}
+}
+
+static void P_ReadRLightlevel (const char *key)
+{
+	int sector, rlightlevel;
+
+	if (sscanf(line, "%s %d %d\n",
+	           string,
+	           &sector,
+	           &rlightlevel) == 3 &&
+	    !strncmp(string, key, MAX_STRING_LEN))
+	{
+		sectors[sector].rlightlevel = (short)rlightlevel;
+	}
+}
+
 // buttonlist[]
 
 extern void P_StartButton (line_t *line, bwhere_e w, int texture, int time);
@@ -449,6 +483,7 @@ static const extsavegdata_t extsavegdata[] =
 	{"fireflicker", P_WriteFireFlicker, P_ReadFireFlicker, 1},
 	{"soundtarget", P_WriteSoundTarget, P_ReadSoundTarget, 1},
 	{"oldspecial", P_WriteOldSpecial, P_ReadOldSpecial, 1},
+	{"rlightlevel", P_WriteRLightlevel, P_ReadRLightlevel, 1},
 	{"button", P_WriteButton, P_ReadButton, 1},
 	{"braintarget", P_WriteBrainTarget, P_ReadBrainTarget, 1},
 	{"markpoints", P_WriteMarkPoints, P_ReadMarkPoints, 1},
