@@ -56,7 +56,6 @@ int			validcount = 1;
 
 
 lighttable_t*		fixedcolormap;
-extern lighttable_t**	walllights;
 
 int			centerx;
 int			centery;
@@ -934,6 +933,8 @@ void R_ExecuteSetViewSize (void)
 
     // [crispy] forcefully initialize the status bar backing screen
     ST_refreshBackground(true);
+
+    pspr_interp = false; // interpolate weapon bobbing
 }
 
 boolean goobers_mode = false;
@@ -1105,7 +1106,6 @@ void R_SetupFrame (player_t* player)
 void R_RenderPlayerView (player_t* player)
 {	
     extern void V_DrawFilledBox (int x, int y, int w, int h, int c);
-    extern void R_InterpolateTextureOffsets (void);
 
     R_SetupFrame (player);
 
