@@ -148,14 +148,17 @@
 #define AMSTR_FOLLOWON		"FOLLOW MODE ON"
 #define AMSTR_FOLLOWOFF		"FOLLOW MODE OFF"
 
-#define AMSTR_GRIDON		"Grid ON"
-#define AMSTR_GRIDOFF		"Grid OFF"
+#define AMSTR_GRIDON		"GRID ON"
+#define AMSTR_GRIDOFF		"GRID OFF"
 
 #define AMSTR_MARKEDSPOT	"Marked Spot"
 #define AMSTR_MARKSCLEARED	"All Marks Cleared"
 
-#define AMSTR_OVERLAYON		"Overlay Mode ON"
-#define AMSTR_OVERLAYOFF	"Overlay Mode OFF"
+#define AMSTR_OVERLAYON		"OVERLAY MODE ON"
+#define AMSTR_OVERLAYOFF	"OVERLAY MODE OFF"
+
+#define AMSTR_ROTATEON		"ROTATE MODE ON"
+#define AMSTR_ROTATEOFF		"ROTATE MODE OFF"
 
 //---------------------------------------------------------------------------
 //
