@@ -248,10 +248,6 @@ fixed_t xspeed[8] =
 fixed_t yspeed[8] =
     { 0, 47000, FRACUNIT, 47000, 0, -47000, -FRACUNIT, -47000 };
 
-#define	MAXSPECIALCROSS		8
-extern line_t **spechit; // [crispy] remove SPECHIT limit
-extern int numspechit;
-
 boolean P_Move(mobj_t * actor)
 {
     fixed_t tryx, tryy;
@@ -613,7 +609,7 @@ boolean P_LookForPlayers(mobj_t * actor, boolean allaround)
 ==============
 */
 
-void A_Look(mobj_t * actor)
+void A_Look(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *targ;
 
@@ -682,7 +678,7 @@ void A_Look(mobj_t * actor)
 ==============
 */
 
-void A_Chase(mobj_t * actor)
+void A_Chase(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     int delta;
 
@@ -711,7 +707,7 @@ void A_Chase(mobj_t * actor)
 //
     if (actor->movedir < 8)
     {
-        actor->angle &= (7 << 29);
+        actor->angle &= (7u << 29);
         delta = actor->angle - (actor->movedir << 29);
         if (delta > 0)
         {
@@ -813,7 +809,7 @@ void A_Chase(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_FaceTarget(mobj_t * actor)
+void A_FaceTarget(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (!actor->target)
     {
@@ -834,7 +830,7 @@ void A_FaceTarget(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_Pain(mobj_t * actor)
+void A_Pain(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (actor->info->painsound)
     {
@@ -848,7 +844,7 @@ void A_Pain(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_DripBlood(mobj_t * actor)
+void A_DripBlood(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     int r1,r2;
@@ -870,7 +866,7 @@ void A_DripBlood(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_KnightAttack(mobj_t * actor)
+void A_KnightAttack(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (!actor->target)
     {
@@ -899,7 +895,7 @@ void A_KnightAttack(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_ImpExplode(mobj_t * actor)
+void A_ImpExplode(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
 
@@ -923,7 +919,7 @@ void A_ImpExplode(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_BeastPuff(mobj_t * actor)
+void A_BeastPuff(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (P_Random() > 64)
     {
@@ -943,7 +939,7 @@ void A_BeastPuff(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_ImpMeAttack(mobj_t * actor)
+void A_ImpMeAttack(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (!actor->target)
     {
@@ -962,7 +958,7 @@ void A_ImpMeAttack(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_ImpMsAttack(mobj_t * actor)
+void A_ImpMsAttack(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *dest;
     angle_t an;
@@ -976,7 +972,7 @@ void A_ImpMsAttack(mobj_t * actor)
     dest = actor->target;
     actor->flags |= MF_SKULLFLY;
     S_StartSound(actor, actor->info->attacksound);
-    A_FaceTarget(actor);
+    A_FaceTarget(actor, player, psp);
     an = actor->angle >> ANGLETOFINESHIFT;
     actor->momx = FixedMul(12 * FRACUNIT, finecosine[an]);
     actor->momy = FixedMul(12 * FRACUNIT, finesine[an]);
@@ -997,7 +993,7 @@ void A_ImpMsAttack(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_ImpMsAttack2(mobj_t * actor)
+void A_ImpMsAttack2(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (!actor->target)
     {
@@ -1018,7 +1014,7 @@ void A_ImpMsAttack2(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_ImpDeath(mobj_t * actor)
+void A_ImpDeath(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags &= ~MF_SOLID;
     actor->flags2 |= MF2_FOOTCLIP;
@@ -1034,7 +1030,7 @@ void A_ImpDeath(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_ImpXDeath1(mobj_t * actor)
+void A_ImpXDeath1(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags &= ~MF_SOLID;
     actor->flags |= MF_NOGRAVITY;
@@ -1048,7 +1044,7 @@ void A_ImpXDeath1(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_ImpXDeath2(mobj_t * actor)
+void A_ImpXDeath2(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags &= ~MF_NOGRAVITY;
     if (actor->z <= actor->floorz)
@@ -1112,7 +1108,7 @@ boolean P_UpdateChicken(mobj_t * actor, int tics)
 //
 //----------------------------------------------------------------------------
 
-void A_ChicAttack(mobj_t * actor)
+void A_ChicAttack(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (P_UpdateChicken(actor, 18))
     {
@@ -1134,13 +1130,13 @@ void A_ChicAttack(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_ChicLook(mobj_t * actor)
+void A_ChicLook(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (P_UpdateChicken(actor, 10))
     {
         return;
     }
-    A_Look(actor);
+    A_Look(actor, player, psp);
 }
 
 //----------------------------------------------------------------------------
@@ -1149,13 +1145,13 @@ void A_ChicLook(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_ChicChase(mobj_t * actor)
+void A_ChicChase(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (P_UpdateChicken(actor, 3))
     {
         return;
     }
-    A_Chase(actor);
+    A_Chase(actor, player, psp);
 }
 
 //----------------------------------------------------------------------------
@@ -1164,7 +1160,7 @@ void A_ChicChase(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_ChicPain(mobj_t * actor)
+void A_ChicPain(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (P_UpdateChicken(actor, 10))
     {
@@ -1179,7 +1175,7 @@ void A_ChicPain(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_Feathers(mobj_t * actor)
+void A_Feathers(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     int i;
     int count;
@@ -1211,7 +1207,7 @@ void A_Feathers(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_MummyAttack(mobj_t * actor)
+void A_MummyAttack(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (!actor->target)
     {
@@ -1235,7 +1231,7 @@ void A_MummyAttack(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_MummyAttack2(mobj_t * actor)
+void A_MummyAttack2(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
 
@@ -1263,7 +1259,7 @@ void A_MummyAttack2(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_MummyFX1Seek(mobj_t * actor)
+void A_MummyFX1Seek(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     P_SeekerMissile(actor, ANG1_X * 10, ANG1_X * 20);
 }
@@ -1274,7 +1270,7 @@ void A_MummyFX1Seek(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_MummySoul(mobj_t * mummy)
+void A_MummySoul(mobj_t * mummy, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
 
@@ -1289,10 +1285,10 @@ void A_MummySoul(mobj_t * mummy)
 //
 //----------------------------------------------------------------------------
 
-void A_Sor1Pain(mobj_t * actor)
+void A_Sor1Pain(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->special1.i = 20;       // Number of steps to walk fast
-    A_Pain(actor);
+    A_Pain(actor, player, psp);
 }
 
 //----------------------------------------------------------------------------
@@ -1301,14 +1297,14 @@ void A_Sor1Pain(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_Sor1Chase(mobj_t * actor)
+void A_Sor1Chase(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (actor->special1.i)
     {
         actor->special1.i--;
         actor->tics -= 3;
     }
-    A_Chase(actor);
+    A_Chase(actor, player, psp);
 }
 
 //----------------------------------------------------------------------------
@@ -1319,7 +1315,7 @@ void A_Sor1Chase(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_Srcr1Attack(mobj_t * actor)
+void A_Srcr1Attack(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     fixed_t momz;
@@ -1370,7 +1366,7 @@ void A_Srcr1Attack(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_SorcererRise(mobj_t * actor)
+void A_SorcererRise(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
 
@@ -1387,7 +1383,7 @@ void A_SorcererRise(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void P_DSparilTeleport(mobj_t * actor)
+void P_DSparilTeleport(mobj_t *actor)
 {
     int i;
     fixed_t x;
@@ -1430,7 +1426,7 @@ void P_DSparilTeleport(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_Srcr2Decide(mobj_t * actor)
+void A_Srcr2Decide(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     static int chance[] = {
         192, 120, 120, 120, 64, 64, 32, 16, 0
@@ -1452,7 +1448,7 @@ void A_Srcr2Decide(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_Srcr2Attack(mobj_t * actor)
+void A_Srcr2Attack(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     int chance;
 
@@ -1486,7 +1482,7 @@ void A_Srcr2Attack(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_BlueSpark(mobj_t * actor)
+void A_BlueSpark(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     int i;
     mobj_t *mo;
@@ -1506,7 +1502,7 @@ void A_BlueSpark(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_GenWizard(mobj_t * actor)
+void A_GenWizard(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     mobj_t *fog;
@@ -1531,7 +1527,7 @@ void A_GenWizard(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_Sor2DthInit(mobj_t * actor)
+void A_Sor2DthInit(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->special1.i = 7;        // Animation loop counter
     P_Massacre();               // Kill monsters early
@@ -1543,7 +1539,7 @@ void A_Sor2DthInit(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_Sor2DthLoop(mobj_t * actor)
+void A_Sor2DthLoop(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (--actor->special1.i)
     {                           // Need to loop
@@ -1557,32 +1553,32 @@ void A_Sor2DthLoop(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_SorZap(mobj_t * actor)
+void A_SorZap(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     S_StartSound(NULL, sfx_sorzap);
 }
 
-void A_SorRise(mobj_t * actor)
+void A_SorRise(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     S_StartSound(NULL, sfx_sorrise);
 }
 
-void A_SorDSph(mobj_t * actor)
+void A_SorDSph(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     S_StartSound(NULL, sfx_sordsph);
 }
 
-void A_SorDExp(mobj_t * actor)
+void A_SorDExp(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     S_StartSound(NULL, sfx_sordexp);
 }
 
-void A_SorDBon(mobj_t * actor)
+void A_SorDBon(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     S_StartSound(NULL, sfx_sordbon);
 }
 
-void A_SorSightSnd(mobj_t * actor)
+void A_SorSightSnd(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     S_StartSound(NULL, sfx_sorsit);
 }
@@ -1595,7 +1591,7 @@ void A_SorSightSnd(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_MinotaurAtk1(mobj_t * actor)
+void A_MinotaurAtk1(mobj_t *actor, player_t *player_, pspdef_t *psp)
 {
     player_t *player;
 
@@ -1624,7 +1620,7 @@ void A_MinotaurAtk1(mobj_t * actor)
 
 #define MNTR_CHARGE_SPEED (13*FRACUNIT)
 
-void A_MinotaurDecide(mobj_t * actor)
+void A_MinotaurDecide(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     angle_t angle;
     mobj_t *target;
@@ -1645,7 +1641,7 @@ void A_MinotaurDecide(mobj_t * actor)
         // Don't call the state function right away
         P_SetMobjStateNF(actor, S_MNTR_ATK4_1);
         actor->flags |= MF_SKULLFLY;
-        A_FaceTarget(actor);
+        A_FaceTarget(actor, player, psp);
         angle = actor->angle >> ANGLETOFINESHIFT;
         actor->momx = FixedMul(MNTR_CHARGE_SPEED, finecosine[angle]);
         actor->momy = FixedMul(MNTR_CHARGE_SPEED, finesine[angle]);
@@ -1659,7 +1655,7 @@ void A_MinotaurDecide(mobj_t * actor)
     }
     else
     {                           // Swing attack
-        A_FaceTarget(actor);
+        A_FaceTarget(actor, player, psp);
         // Don't need to call P_SetMobjState because the current state
         // falls through to the swing attack
     }
@@ -1671,7 +1667,7 @@ void A_MinotaurDecide(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_MinotaurCharge(mobj_t * actor)
+void A_MinotaurCharge(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *puff;
 
@@ -1696,7 +1692,7 @@ void A_MinotaurCharge(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_MinotaurAtk2(mobj_t * actor)
+void A_MinotaurAtk2(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     angle_t angle;
@@ -1733,7 +1729,7 @@ void A_MinotaurAtk2(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_MinotaurAtk3(mobj_t * actor)
+void A_MinotaurAtk3(mobj_t *actor, player_t *player_, pspdef_t *psp)
 {
     mobj_t *mo;
     player_t *player;
@@ -1771,7 +1767,7 @@ void A_MinotaurAtk3(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_MntrFloorFire(mobj_t * actor)
+void A_MntrFloorFire(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     int r1, r2;
@@ -1794,7 +1790,7 @@ void A_MntrFloorFire(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_BeastAttack(mobj_t * actor)
+void A_BeastAttack(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (!actor->target)
     {
@@ -1815,7 +1811,7 @@ void A_BeastAttack(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_HeadAttack(mobj_t * actor)
+void A_HeadAttack(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     int i;
     mobj_t *fire;
@@ -1837,7 +1833,7 @@ void A_HeadAttack(mobj_t * actor)
     {
         return;
     }
-    A_FaceTarget(actor);
+    A_FaceTarget(actor, player, psp);
     if (P_CheckMeleeRange(actor))
     {
         P_DamageMobj(target, actor, actor, HITDICE(6));
@@ -1896,7 +1892,7 @@ void A_HeadAttack(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_WhirlwindSeek(mobj_t * actor)
+void A_WhirlwindSeek(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->health -= 3;
     if (actor->health < 0)
@@ -1925,7 +1921,7 @@ void A_WhirlwindSeek(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_HeadIceImpact(mobj_t * ice)
+void A_HeadIceImpact(mobj_t * ice, player_t *player, pspdef_t *psp)
 {
     unsigned int i;
     angle_t angle;
@@ -1951,7 +1947,7 @@ void A_HeadIceImpact(mobj_t * ice)
 //
 //----------------------------------------------------------------------------
 
-void A_HeadFireGrow(mobj_t * fire)
+void A_HeadFireGrow(mobj_t * fire, player_t *player, pspdef_t *psp)
 {
     fire->health--;
     fire->z += 9 * FRACUNIT;
@@ -1968,7 +1964,7 @@ void A_HeadFireGrow(mobj_t * fire)
 //
 //----------------------------------------------------------------------------
 
-void A_SnakeAttack(mobj_t * actor)
+void A_SnakeAttack(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (!actor->target)
     {
@@ -1976,7 +1972,7 @@ void A_SnakeAttack(mobj_t * actor)
         return;
     }
     S_StartSound(actor, actor->info->attacksound);
-    A_FaceTarget(actor);
+    A_FaceTarget(actor, player, psp);
     P_SpawnMissile(actor, actor->target, MT_SNAKEPRO_A);
 }
 
@@ -1986,7 +1982,7 @@ void A_SnakeAttack(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_SnakeAttack2(mobj_t * actor)
+void A_SnakeAttack2(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (!actor->target)
     {
@@ -1994,7 +1990,7 @@ void A_SnakeAttack2(mobj_t * actor)
         return;
     }
     S_StartSound(actor, actor->info->attacksound);
-    A_FaceTarget(actor);
+    A_FaceTarget(actor, player, psp);
     P_SpawnMissile(actor, actor->target, MT_SNAKEPRO_B);
 }
 
@@ -2004,7 +2000,7 @@ void A_SnakeAttack2(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_ClinkAttack(mobj_t * actor)
+void A_ClinkAttack(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     int damage;
 
@@ -2026,7 +2022,7 @@ void A_ClinkAttack(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_GhostOff(mobj_t * actor)
+void A_GhostOff(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags &= ~MF_SHADOW;
 }
@@ -2037,9 +2033,9 @@ void A_GhostOff(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_WizAtk1(mobj_t * actor)
+void A_WizAtk1(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
-    A_FaceTarget(actor);
+    A_FaceTarget(actor, player, psp);
     actor->flags &= ~MF_SHADOW;
 }
 
@@ -2049,9 +2045,9 @@ void A_WizAtk1(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_WizAtk2(mobj_t * actor)
+void A_WizAtk2(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
-    A_FaceTarget(actor);
+    A_FaceTarget(actor, player, psp);
     actor->flags |= MF_SHADOW;
 }
 
@@ -2061,7 +2057,7 @@ void A_WizAtk2(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_WizAtk3(mobj_t * actor)
+void A_WizAtk3(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     angle_t angle;
@@ -2094,7 +2090,7 @@ void A_WizAtk3(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_Scream(mobj_t * actor)
+void A_Scream(mobj_t * actor, player_t *player, pspdef_t *psp)
 {
     switch (actor->type)
     {
@@ -2158,7 +2154,7 @@ void P_DropItem(mobj_t * source, mobjtype_t type, int special, int chance)
 //
 //----------------------------------------------------------------------------
 
-void A_NoBlocking(mobj_t * actor)
+void A_NoBlocking(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags &= ~MF_SOLID;
     // Check for monsters dropping things
@@ -2208,7 +2204,7 @@ void A_NoBlocking(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_Explode(mobj_t * actor)
+void A_Explode(mobj_t * actor, player_t *player, pspdef_t *psp)
 {
     int damage;
 
@@ -2238,7 +2234,7 @@ void A_Explode(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_PodPain(mobj_t * actor)
+void A_PodPain(mobj_t * actor, player_t *player, pspdef_t *psp)
 {
     int i;
     int count;
@@ -2268,7 +2264,7 @@ void A_PodPain(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_RemovePod(mobj_t * actor)
+void A_RemovePod(mobj_t * actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
 
@@ -2290,7 +2286,7 @@ void A_RemovePod(mobj_t * actor)
 
 #define MAX_GEN_PODS 16
 
-void A_MakePod(mobj_t * actor)
+void A_MakePod(mobj_t * actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     fixed_t x;
@@ -2351,7 +2347,7 @@ void P_Massacre(void)
 //
 //----------------------------------------------------------------------------
 
-void A_BossDeath(mobj_t * actor)
+void A_BossDeath(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     thinker_t *think;
@@ -2400,7 +2396,7 @@ void A_BossDeath(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_ESound(mobj_t * mo)
+void A_ESound(mobj_t *mo, player_t *player, pspdef_t *psp)
 {
     int sound = sfx_None;
 
@@ -2424,7 +2420,7 @@ void A_ESound(mobj_t * mo)
 //
 //----------------------------------------------------------------------------
 
-void A_SpawnTeleGlitter(mobj_t * actor)
+void A_SpawnTeleGlitter(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     int r1, r2;
@@ -2443,7 +2439,7 @@ void A_SpawnTeleGlitter(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_SpawnTeleGlitter2(mobj_t * actor)
+void A_SpawnTeleGlitter2(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     int r1, r2;
@@ -2462,7 +2458,7 @@ void A_SpawnTeleGlitter2(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_AccTeleGlitter(mobj_t * actor)
+void A_AccTeleGlitter(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (++actor->health > 35)
     {
@@ -2476,7 +2472,7 @@ void A_AccTeleGlitter(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_InitKeyGizmo(mobj_t * gizmo)
+void A_InitKeyGizmo(mobj_t * gizmo, player_t *player, pspdef_t *psp)
 {
     mobj_t *mo;
     statenum_t state = S_NULL;
@@ -2506,7 +2502,7 @@ void A_InitKeyGizmo(mobj_t * gizmo)
 //
 //----------------------------------------------------------------------------
 
-void A_VolcanoSet(mobj_t * volcano)
+void A_VolcanoSet(mobj_t * volcano, player_t *player, pspdef_t *psp)
 {
     volcano->tics = 105 + (P_Random() & 127);
 }
@@ -2517,7 +2513,7 @@ void A_VolcanoSet(mobj_t * volcano)
 //
 //----------------------------------------------------------------------------
 
-void A_VolcanoBlast(mobj_t * volcano)
+void A_VolcanoBlast(mobj_t * volcano, player_t *player, pspdef_t *psp)
 {
     int i;
     int count;
@@ -2546,7 +2542,7 @@ void A_VolcanoBlast(mobj_t * volcano)
 //
 //----------------------------------------------------------------------------
 
-void A_VolcBallImpact(mobj_t * ball)
+void A_VolcBallImpact(mobj_t * ball, player_t *player, pspdef_t *psp)
 {
     unsigned int i;
     mobj_t *tiny;
@@ -2580,7 +2576,7 @@ void A_VolcBallImpact(mobj_t * ball)
 //
 //----------------------------------------------------------------------------
 
-void A_SkullPop(mobj_t * actor)
+void A_SkullPop(mobj_t *actor, player_t *player_, pspdef_t *psp)
 {
     mobj_t *mo;
     player_t *player;
@@ -2617,7 +2613,7 @@ void A_SkullPop(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_CheckSkullFloor(mobj_t * actor)
+void A_CheckSkullFloor(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (actor->z <= actor->floorz)
     {
@@ -2631,7 +2627,7 @@ void A_CheckSkullFloor(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_CheckSkullDone(mobj_t * actor)
+void A_CheckSkullDone(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (actor->special2.i == 666)
     {
@@ -2645,7 +2641,7 @@ void A_CheckSkullDone(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_CheckBurnGone(mobj_t * actor)
+void A_CheckBurnGone(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (actor->special2.i == 666)
     {
@@ -2659,7 +2655,7 @@ void A_CheckBurnGone(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_FreeTargMobj(mobj_t * mo)
+void A_FreeTargMobj(mobj_t * mo, player_t *player, pspdef_t *psp)
 {
     mo->momx = mo->momy = mo->momz = 0;
     mo->z = mo->ceilingz + 4 * FRACUNIT;
@@ -2679,7 +2675,7 @@ void A_FreeTargMobj(mobj_t * mo)
 mobj_t *bodyque[BODYQUESIZE];
 int bodyqueslot;
 
-void A_AddPlayerCorpse(mobj_t * actor)
+void A_AddPlayerCorpse(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     if (bodyqueslot >= BODYQUESIZE)
     {                           // Too many player corpses - remove an old one
@@ -2695,7 +2691,7 @@ void A_AddPlayerCorpse(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_FlameSnd(mobj_t * actor)
+void A_FlameSnd(mobj_t * actor, player_t *player, pspdef_t *psp)
 {
     S_StartSound(actor, sfx_hedat1);    // Burn sound
 }
@@ -2706,7 +2702,7 @@ void A_FlameSnd(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_HideThing(mobj_t * actor)
+void A_HideThing(mobj_t * actor, player_t *player, pspdef_t *psp)
 {
     //P_UnsetThingPosition(actor);
     actor->flags2 |= MF2_DONTDRAW;
@@ -2718,7 +2714,7 @@ void A_HideThing(mobj_t * actor)
 //
 //----------------------------------------------------------------------------
 
-void A_UnHideThing(mobj_t * actor)
+void A_UnHideThing(mobj_t * actor, player_t *player, pspdef_t *psp)
 {
     //P_SetThingPosition(actor);
     actor->flags2 &= ~MF2_DONTDRAW;
