@@ -26,6 +26,7 @@
 #include "m_bbox.h"
 #include "p_local.h"
 #include "s_sound.h"
+#include "p_extnodes.h"
 
 void P_SpawnMapThing(mapthing_t * mthing);
 
@@ -62,6 +63,20 @@ mapthing_t deathmatchstarts[10], *deathmatch_p;
 mapthing_t playerstarts[MAXPLAYERS];
 boolean playerstartsingame[MAXPLAYERS];
 
+// [crispy] recalculate seg offsets
+// adapted from prboom-plus/src/p_setup.c:474-482
+fixed_t GetOffset(vertex_t *v1, vertex_t *v2)
+{
+    fixed_t dx, dy;
+    fixed_t r;
+
+    dx = (v1->x - v2->x)>>FRACBITS;
+    dy = (v1->y - v2->y)>>FRACBITS;
+    r = (fixed_t)(sqrt(dx*dx + dy*dy))<<FRACBITS;
+
+    return r;
+}
+
 /*
 =================
 =
@@ -124,17 +139,19 @@ void P_LoadSegs(int lump)
     li = segs;
     for (i = 0; i < numsegs; i++, li++, ml++)
     {
-        li->v1 = &vertexes[SHORT(ml->v1)];
-        li->v2 = &vertexes[SHORT(ml->v2)];
+        li->v1 = &vertexes[(unsigned short)SHORT(ml->v1)]; // [crispy] extended nodes
+        li->v2 = &vertexes[(unsigned short)SHORT(ml->v2)]; // [crispy] extended nodes
 
         li->angle = (SHORT(ml->angle)) << 16;
         li->offset = (SHORT(ml->offset)) << 16;
-        linedef = SHORT(ml->linedef);
+        linedef = (unsigned short)SHORT(ml->linedef); // [crispy] extended nodes
         ldef = &lines[linedef];
         li->linedef = ldef;
         side = SHORT(ml->side);
         li->sidedef = &sides[ldef->sidenum[side]];
         li->frontsector = sides[ldef->sidenum[side]].sector;
+        // [crispy] recalculate
+        li->offset = GetOffset(li->v1, (ml->side ? ldef->v2 : ldef->v1));
         if (ldef->flags & ML_TWOSIDED)
             li->backsector = sides[ldef->sidenum[side ^ 1]].sector;
         else
@@ -144,6 +161,45 @@ void P_LoadSegs(int lump)
     W_ReleaseLumpNum(lump);
 }
 
+// [crispy] fix long wall wobble
+
+static angle_t anglediff(angle_t a, angle_t b)
+{
+    if (b > a)
+        return anglediff(b, a);
+
+    if (a - b < ANG180)
+        return a - b;
+    else // [crispy] wrap around
+        return b - a;
+}
+
+static void P_SegLengths(void)
+{
+    int i;
+
+    for (i = 0; i < numsegs; i++)
+    {
+        seg_t *const li = &segs[i];
+        int64_t dx, dy;
+
+        dx = li->v2->r_x - li->v1->r_x;
+        dy = li->v2->r_y - li->v1->r_y;
+
+        li->length = (uint32_t)(sqrt((double)dx * dx + (double)dy * dy) / 2);
+
+        // [crispy] re-calculate angle used for rendering
+        viewx = li->v1->r_x;
+        viewy = li->v1->r_y;
+        li->r_angle = R_PointToAngleCrispy(li->v2->r_x, li->v2->r_y);
+        // [crispy] more than just a little adjustment?
+        // back to the original angle then
+        if (anglediff(li->r_angle, li->angle) > ANG60/2)
+        {
+            li->r_angle = li->angle;
+        }
+    }
+}
 
 /*
 =================
@@ -169,8 +225,8 @@ void P_LoadSubsectors(int lump)
     ss = subsectors;
     for (i = 0; i < numsubsectors; i++, ss++, ms++)
     {
-        ss->numlines = SHORT(ms->numsegs);
-        ss->firstline = SHORT(ms->firstseg);
+        ss->numlines = (unsigned short)SHORT(ms->numsegs); // [crispy] extended nodes
+        ss->firstline = (unsigned short)SHORT(ms->firstseg); // [crispy] extended nodes
     }
 
     W_ReleaseLumpNum(lump);
@@ -210,6 +266,9 @@ void P_LoadSectors(int lump)
         ss->tag = SHORT(ms->tag);
         ss->thinglist = NULL;
 
+        // [crispy] WiggleFix: [kb] for R_FixWiggle()
+        ss->cachedheight = 0;
+
         // [AM] Sector interpolation.  Even if we're
         //      not running uncapped, the renderer still
         //      uses this data.
@@ -254,7 +313,23 @@ void P_LoadNodes(int lump)
         no->dy = SHORT(mn->dy) << FRACBITS;
         for (j = 0; j < 2; j++)
         {
-            no->children[j] = SHORT(mn->children[j]);
+            no->children[j] = (unsigned short)SHORT(mn->children[j]); // [crispy] extended nodes
+
+            // [crispy] add support for extended nodes
+            // from prboom-plus/src/p_setup.c:937-957
+            if (no->children[j] == NO_INDEX)
+                no->children[j] = -1;
+            else
+            if (no->children[j] & NF_SUBSECTOR_VANILLA)
+            {
+                no->children[j] &= ~NF_SUBSECTOR_VANILLA;
+
+                if (no->children[j] >= numsubsectors)
+                    no->children[j] = 0;
+
+                no->children[j] |= NF_SUBSECTOR;
+            }
+
             for (k = 0; k < 4; k++)
                 no->bbox[j][k] = SHORT(mn->bbox[j][k]) << FRACBITS;
         }
@@ -338,11 +413,11 @@ void P_LoadLineDefs(int lump)
     ld = lines;
     for (i = 0; i < numlines; i++, mld++, ld++)
     {
-        ld->flags = SHORT(mld->flags);
+        ld->flags = (unsigned short)SHORT(mld->flags); // [crispy] extended nodes
         ld->special = SHORT(mld->special);
         ld->tag = SHORT(mld->tag);
-        v1 = ld->v1 = &vertexes[SHORT(mld->v1)];
-        v2 = ld->v2 = &vertexes[SHORT(mld->v2)];
+        v1 = ld->v1 = &vertexes[(unsigned short)SHORT(mld->v1)]; // [crispy] extended nodes
+        v2 = ld->v2 = &vertexes[(unsigned short)SHORT(mld->v2)]; // [crispy] extended nodes
         ld->dx = v2->x - v1->x;
         ld->dy = v2->y - v1->y;
         if (!ld->dx)
@@ -379,11 +454,11 @@ void P_LoadLineDefs(int lump)
         }
         ld->sidenum[0] = SHORT(mld->sidenum[0]);
         ld->sidenum[1] = SHORT(mld->sidenum[1]);
-        if (ld->sidenum[0] != -1)
+        if (ld->sidenum[0] != NO_INDEX) // [crispy] extended nodes
             ld->frontsector = sides[ld->sidenum[0]].sector;
         else
             ld->frontsector = 0;
-        if (ld->sidenum[1] != -1)
+        if (ld->sidenum[1] != NO_INDEX) // [crispy] extended nodes
             ld->backsector = sides[ld->sidenum[1]].sector;
         else
             ld->backsector = 0;
@@ -643,6 +718,7 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
     char lumpname[9];
     int lumpnum;
     mobj_t *mobj;
+    mapformat_t crispy_mapformat;
 
     totalkills = totalitems = totalsecret = 0;
     for (i = 0; i < MAXPLAYERS; i++)
@@ -679,6 +755,9 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
 
     lumpnum = W_GetNumForName(lumpname);
 
+    // [crispy] check and log map and nodes format
+    crispy_mapformat = P_CheckMapFormat(lumpnum);
+
 // note: most of this ordering is important     
     P_LoadBlockMap(lumpnum + ML_BLOCKMAP);
     P_LoadVertexes(lumpnum + ML_VERTEXES);
@@ -686,9 +765,23 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
     P_LoadSideDefs(lumpnum + ML_SIDEDEFS);
 
     P_LoadLineDefs(lumpnum + ML_LINEDEFS);
+
+    if (crispy_mapformat & (MFMT_ZDBSPX | MFMT_ZDBSPZ))
+    {
+        P_LoadNodes_ZDBSP(lumpnum + ML_NODES, crispy_mapformat & MFMT_ZDBSPZ);
+    }
+    else if (crispy_mapformat & MFMT_DEEPBSP)
+    {
+        P_LoadSubsectors_DeePBSP(lumpnum + ML_SSECTORS);
+        P_LoadNodes_DeePBSP(lumpnum + ML_NODES);
+        P_LoadSegs_DeePBSP(lumpnum + ML_SEGS);
+    }
+    else
+    {
     P_LoadSubsectors(lumpnum + ML_SSECTORS);
     P_LoadNodes(lumpnum + ML_NODES);
     P_LoadSegs(lumpnum + ML_SEGS);
+    }
 
     rejectmatrix = W_CacheLumpNum(lumpnum + ML_REJECT, PU_LEVEL);
     P_GroupLines();
@@ -696,6 +789,9 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
     // [crispy] remove slime trails
     P_RemoveSlimeTrails();
 
+    // [crispy] fix long wall wobble
+    P_SegLengths();
+
     bodyqueslot = 0;
     deathmatch_p = deathmatchstarts;
     P_InitAmbientSound();
