@@ -209,8 +209,10 @@ void R_ClearClipSegs(void)
 void R_CheckInterpolateSector(sector_t* sector)
 {
     if (crispy->uncapped &&
-        // Only if we moved the sector last tic.
-        sector->oldgametic == gametic - 1)
+        // Only if we moved the sector last tic ...
+        sector->oldgametic == gametic - 1 &&
+        // ... and it has a thinker associated with it.
+        sector->specialdata)
     {
         // Interpolate between current and last floor/ceiling position.
         if (sector->floorheight != sector->oldfloorheight)
@@ -251,8 +253,8 @@ void R_AddLine(seg_t * line)
 // OPTIMIZE: quickly reject orthogonal back sides
 
     // [crispy] remove slime trails
-    angle1 = R_PointToAngle(line->v1->r_x, line->v1->r_y);
-    angle2 = R_PointToAngle(line->v2->r_x, line->v2->r_y);
+    angle1 = R_PointToAngleCrispy(line->v1->r_x, line->v1->r_y);
+    angle2 = R_PointToAngleCrispy(line->v2->r_x, line->v2->r_y);
 
 //
 // clip to view edges
@@ -389,8 +391,8 @@ boolean R_CheckBBox(fixed_t * bspcoord)
 //
 // check clip list for an open space
 //      
-    angle1 = R_PointToAngle(x1, y1) - viewangle;
-    angle2 = R_PointToAngle(x2, y2) - viewangle;
+    angle1 = R_PointToAngleCrispy(x1, y1) - viewangle;
+    angle2 = R_PointToAngleCrispy(x2, y2) - viewangle;
 
     span = angle1 - angle2;
     if (span >= ANG180)
