@@ -30,7 +30,6 @@ int viewangleoffset;
 int validcount = 1;             // increment every time a check is made
 
 lighttable_t *fixedcolormap;
-extern lighttable_t **walllights;
 
 int centerx, centery;
 fixed_t centerxfrac, centeryfrac;
@@ -193,15 +192,19 @@ int R_PointOnSegSide(fixed_t x, fixed_t y, seg_t * line)
 }
 
 
+// [crispy] turned into a general R_PointToAngle() flavor
+// called with either slope_div = SlopeDivCrispy() from R_PointToAngleCrispy()
+// or slope_div = SlopeDiv() else
 /*
 ===============================================================================
 =
-= R_PointToAngle
+= R_PointToAngleSlope
 =
 ===============================================================================
 */
 
-angle_t R_PointToAngle(fixed_t x, fixed_t y)
+angle_t R_PointToAngleSlope(fixed_t x, fixed_t y,
+            int (*slope_div) (unsigned int num, unsigned int den))
 {
     x -= viewx;
     y -= viewy;
@@ -212,17 +215,17 @@ angle_t R_PointToAngle(fixed_t x, fixed_t y)
         if (y >= 0)
         {                       // y>= 0
             if (x > y)
-                return tantoangle[SlopeDiv(y, x)];      // octant 0
+                return tantoangle[slope_div(y, x)];      // octant 0
             else
-                return ANG90 - 1 - tantoangle[SlopeDiv(x, y)];  // octant 1
+                return ANG90 - 1 - tantoangle[slope_div(x, y)];  // octant 1
         }
         else
         {                       // y<0
             y = -y;
             if (x > y)
-                return -tantoangle[SlopeDiv(y, x)];     // octant 8
+                return -tantoangle[slope_div(y, x)];     // octant 8
             else
-                return ANG270 + tantoangle[SlopeDiv(x, y)];     // octant 7
+                return ANG270 + tantoangle[slope_div(x, y)];     // octant 7
         }
     }
     else
@@ -231,17 +234,17 @@ angle_t R_PointToAngle(fixed_t x, fixed_t y)
         if (y >= 0)
         {                       // y>= 0
             if (x > y)
-                return ANG180 - 1 - tantoangle[SlopeDiv(y, x)]; // octant 3
+                return ANG180 - 1 - tantoangle[slope_div(y, x)]; // octant 3
             else
-                return ANG90 + tantoangle[SlopeDiv(x, y)];      // octant 2
+                return ANG90 + tantoangle[slope_div(x, y)];      // octant 2
         }
         else
         {                       // y<0
             y = -y;
             if (x > y)
-                return ANG180 + tantoangle[SlopeDiv(y, x)];     // octant 4
+                return ANG180 + tantoangle[slope_div(y, x)];     // octant 4
             else
-                return ANG270 - 1 - tantoangle[SlopeDiv(x, y)]; // octant 5
+                return ANG270 - 1 - tantoangle[slope_div(x, y)]; // octant 5
         }
     }
 
@@ -249,11 +252,37 @@ angle_t R_PointToAngle(fixed_t x, fixed_t y)
 }
 
 
+angle_t R_PointToAngle(fixed_t x, fixed_t y)
+{
+    return R_PointToAngleSlope(x, y, SlopeDiv);
+}
+
+// [crispy] overflow-safe R_PointToAngle() flavor
+// called only from R_CheckBBox(), R_AddLine() and P_SegLengths()
+angle_t R_PointToAngleCrispy(fixed_t x, fixed_t y)
+{
+    // [crispy] fix overflows for very long distances
+    int64_t y_viewy = (int64_t)y - viewy;
+    int64_t x_viewx = (int64_t)x - viewx;
+
+    // [crispy] the worst that could happen is e.g. INT_MIN-INT_MAX = 2*INT_MIN
+    if (x_viewx < INT_MIN || x_viewx > INT_MAX ||
+        y_viewy < INT_MIN || y_viewy > INT_MAX)
+    {
+        // [crispy] preserving the angle by halfing the distance in both directions
+        x = x_viewx / 2 + viewx;
+        y = y_viewy / 2 + viewy;
+    }
+
+    return R_PointToAngleSlope(x, y, SlopeDivCrispy);
+}
+
 angle_t R_PointToAngle2(fixed_t x1, fixed_t y1, fixed_t x2, fixed_t y2)
 {
     viewx = x1;
     viewy = y1;
-    return R_PointToAngle(x2, y2);
+    // [crispy] R_PointToAngle2() is never called during rendering
+    return R_PointToAngleSlope(x2, y2, SlopeDiv);
 }
 
 
@@ -312,6 +341,9 @@ void R_InitPointToAngle(void)
 
 //=============================================================================
 
+// [crispy] WiggleFix: move R_ScaleFromGlobalAngle function to r_segs.c, above
+// R_StoreWallRange
+#if 0
 /*
 ================
 =
@@ -363,6 +395,7 @@ fixed_t R_ScaleFromGlobalAngle(angle_t visangle)
 
     return scale;
 }
+#endif
 
 
 // [AM] Interpolate between two angles.
@@ -703,6 +736,8 @@ void R_ExecuteSetViewSize(void)
 
     // [crispy] Redraw status bar, needed for widescreen HUD
     SB_ForceRedraw();
+
+    pspr_interp = false; // [crispy]
 }
 
 
@@ -888,8 +923,6 @@ void R_SetupFrame(player_t * player)
 
 void R_RenderPlayerView(player_t * player)
 {
-    // [crispy] Smooth texture scrolling
-    extern void R_InterpolateTextureOffsets(void);
     extern boolean automapactive;
 
     R_SetupFrame(player);
