@@ -19,7 +19,6 @@
 #include "i_system.h"
 #include "p_local.h"
 
-extern fixed_t FloatBobOffsets[64];
 
 //==================================================================
 //==================================================================
@@ -189,8 +188,9 @@ result_e T_MovePlane(sector_t * sector, fixed_t speed,
 //      MOVE A FLOOR TO IT'S DESTINATION (UP OR DOWN)
 //
 //==================================================================
-void T_MoveFloor(floormove_t * floor)
+void T_MoveFloor(thinker_t *thinker)
 {
+    floormove_t *floor = (floormove_t *) thinker;
     result_e res;
 
     if (floor->resetDelayCount)
@@ -659,8 +659,9 @@ int EV_BuildStairs(line_t * line, byte * args, int direction,
 //
 //=========================================================================
 
-void T_BuildPillar(pillar_t * pillar)
+void T_BuildPillar(thinker_t *thinker)
 {
+	pillar_t *pillar = (pillar_t *) thinker;
     result_e res1;
     result_e res2;
 
@@ -867,8 +868,9 @@ int EV_FloorCrushStop(line_t * line, byte * args)
 #define WGLSTATE_STABLE 2
 #define WGLSTATE_REDUCE 3
 
-void T_FloorWaggle(floorWaggle_t * waggle)
+void T_FloorWaggle(thinker_t *thinker)
 {
+	floorWaggle_t *waggle = (floorWaggle_t *) thinker;
     switch (waggle->state)
     {
         case WGLSTATE_EXPAND:
