@@ -568,8 +568,6 @@ boolean P_GiveArmor(player_t * player, armortype_t armortype, int amount)
     int hits;
     int totalArmor;
 
-    extern int ArmorMax[NUMCLASSES];
-
     if (amount == -1)
     {
         hits = ArmorIncrement[player->class][armortype];
@@ -895,7 +893,7 @@ static void SetDormantArtifact(mobj_t * arti)
 //
 //---------------------------------------------------------------------------
 
-void A_RestoreArtifact(mobj_t * arti)
+void A_RestoreArtifact(mobj_t *arti, player_t *player, pspdef_t *psp)
 {
     arti->flags |= MF_SPECIAL;
     P_SetMobjState(arti, arti->info->spawnstate);
@@ -910,7 +908,7 @@ void A_RestoreArtifact(mobj_t * arti)
 //
 //---------------------------------------------------------------------------
 
-void A_RestoreSpecialThing1(mobj_t * thing)
+void A_RestoreSpecialThing1(mobj_t * thing, player_t *player, pspdef_t *psp)
 {
     thing->flags2 &= ~MF2_DONTDRAW;
     S_StartSound(thing, SFX_RESPAWN);
@@ -922,7 +920,7 @@ void A_RestoreSpecialThing1(mobj_t * thing)
 //
 //---------------------------------------------------------------------------
 
-void A_RestoreSpecialThing2(mobj_t * thing)
+void A_RestoreSpecialThing2(mobj_t * thing, player_t *player, pspdef_t *psp)
 {
     thing->flags |= MF_SPECIAL;
     P_SetMobjState(thing, thing->info->spawnstate);
