@@ -21,7 +21,6 @@
 #include "p_local.h"
 #include "s_sound.h"
 
-void P_PlayerNextArtifact(player_t * player);
 
 // Macros
 
@@ -63,6 +62,9 @@ int PStateAttackEnd[NUMCLASSES] = {
 
 int ArmorMax[NUMCLASSES] = { 20, 18, 16, 1 };
 
+// [crispy] variable player view bob
+static const fixed_t crispy_bobfactor[3] = {4, 3, 0};
+
 /*
 ==================
 =
@@ -124,6 +126,9 @@ void P_CalcHeight(player_t * player)
         player->bob = FRACUNIT / 2;
     }
 
+    // [crispy] variable player view bob
+    player->bob2 = crispy_bobfactor[crispy->bobfactor] * player->bob / 4;
+
     if ((player->cheats & CF_NOMOMENTUM))
     {
         player->viewz = player->mo->z + VIEWHEIGHT;
@@ -134,7 +139,8 @@ void P_CalcHeight(player_t * player)
     }
 
     angle = (FINEANGLES / 20 * leveltime) & FINEMASK;
-    bob = FixedMul(player->bob / 2, finesine[angle]);
+    // [crispy] variable player view bob
+    bob = FixedMul(player->bob2 / 2, finesine[angle]);
 
 //
 // move viewheight
@@ -1655,7 +1661,7 @@ boolean P_UseArtifact(player_t * player, artitype_t arti)
 //
 //============================================================================
 
-void A_SpeedFade(mobj_t * actor)
+void A_SpeedFade(mobj_t *actor, player_t *player, pspdef_t *psp)
 {
     actor->flags |= MF_SHADOW;
     actor->flags &= ~MF_ALTSHADOW;
