@@ -28,7 +28,6 @@ int viewangleoffset;
 int validcount = 1;             // increment every time a check is made
 
 lighttable_t *fixedcolormap;
-extern lighttable_t **walllights;
 
 int centerx, centery;
 fixed_t centerxfrac, centeryfrac;
@@ -314,6 +313,9 @@ void R_InitPointToAngle(void)
 
 //=============================================================================
 
+// [crispy] WiggleFix: move R_ScaleFromGlobalAngle function to r_segs.c, above
+// R_StoreWallRange
+#if 0
 /*
 ================
 =
@@ -365,6 +367,7 @@ fixed_t R_ScaleFromGlobalAngle(angle_t visangle)
 
     return scale;
 }
+#endif
 
 
 // [AM] Interpolate between two angles.
@@ -703,6 +706,8 @@ void R_ExecuteSetViewSize(void)
 
     // [crispy] Redraw status bar needed for widescreen HUD
     SB_ForceRedraw();
+
+    pspr_interp = false; // [crispy]
 }
 
 
@@ -899,7 +904,6 @@ void R_SetupFrame(player_t * player)
 void R_RenderPlayerView(player_t * player)
 {
     extern void PO_InterpolatePolyObjects(void);
-    extern void R_InterpolateTextureOffset(void);
     extern boolean automapactive;
 
     R_SetupFrame(player);
@@ -916,7 +920,7 @@ void R_RenderPlayerView(player_t * player)
 
     NetUpdate();                // check for new console commands
     PO_InterpolatePolyObjects(); // [crispy] Interpolate polyobjects here
-    R_InterpolateTextureOffset(); // [crispy] Smooth texture scrolling
+    R_InterpolateTextureOffsets(); // [crispy] Smooth texture scrolling
 
     // Make displayed player invisible locally
     if (localQuakeHappening[displayplayer] && gamestate == GS_LEVEL)
