@@ -20,6 +20,8 @@
 #include "h2def.h"
 #include "i_system.h"
 #include "r_local.h"
+#include "p_spec.h"
+
 
 // MACROS ------------------------------------------------------------------
 
@@ -33,9 +35,6 @@
 
 // EXTERNAL DATA DECLARATIONS ----------------------------------------------
 
-extern fixed_t Sky1ScrollDelta;
-extern fixed_t Sky2ScrollDelta;
-
 // PUBLIC DATA DEFINITIONS -------------------------------------------------
 
 int Sky1Texture;
@@ -51,13 +50,13 @@ planefunction_t floorfunc, ceilingfunc;
 // Opening
 visplane_t visplanes[MAXVISPLANES], *lastvisplane;
 visplane_t *floorplane, *ceilingplane;
-short openings[MAXOPENINGS], *lastopening;
+int openings[MAXOPENINGS], *lastopening; // [crispy] 32-bit integer math
 
 // Clip values are the solid pixel bounding the range.
 // floorclip start out SCREENHEIGHT
 // ceilingclip starts out -1
-short floorclip[MAXWIDTH];
-short ceilingclip[MAXWIDTH];
+int floorclip[MAXWIDTH]; // [crispy] 32-bit integer math
+int ceilingclip[MAXWIDTH]; // [crispy] 32-bit integer math
 
 // spanstart holds the start of a plane span, initialized to 0
 int spanstart[MAXHEIGHT];
@@ -317,7 +316,7 @@ visplane_t *R_CheckPlane(visplane_t * pl, int start, int stop)
 
     for (x = intrl; x <= intrh; x++)
     {
-        if (pl->top[x] != 0xffff)
+        if (pl->top[x] != 0xffffffffu) // [crispy] hires / 32-bit integer math
         {
             break;
         }
@@ -349,7 +348,8 @@ visplane_t *R_CheckPlane(visplane_t * pl, int start, int stop)
 //
 //==========================================================================
 
-void R_MakeSpans(int x, int t1, int b1, int t2, int b2)
+// [crispy] 32-bit integer math
+void R_MakeSpans(int x, unsigned int t1, unsigned int b1, unsigned int t2, unsigned int b2)
 {
     while (t1 < t2 && t1 <= b1)
     {
@@ -404,9 +404,6 @@ void R_DrawPlanes(void)
     static int interpfactor; // [crispy]
     int heightmask; // [crispy]
 
-    extern byte *ylookup[MAXHEIGHT];
-    extern int columnofs[MAXWIDTH];
-
 #ifdef RANGECHECK
     if (ds_p - drawsegs > MAXDRAWSEGS)
     {
@@ -443,7 +440,7 @@ void R_DrawPlanes(void)
                 {
                     dc_yl = pl->top[x];
                     dc_yh = pl->bottom[x];
-                    if (dc_yl <= dc_yh)
+                    if ((unsigned) dc_yl <= dc_yh) // [crispy] 32-bit integer math
                     {
                         count = dc_yh - dc_yl;
                         if (count < 0)
@@ -525,7 +522,7 @@ void R_DrawPlanes(void)
                 {
                     dc_yl = pl->top[x];
                     dc_yh = pl->bottom[x];
-                    if (dc_yl <= dc_yh)
+                    if ((unsigned) dc_yl <= dc_yh) // [crispy] 32-bit integer math
                     {
                         count = dc_yh - dc_yl;
                         if (count < 0)
@@ -689,8 +686,8 @@ void R_DrawPlanes(void)
         }
         planezlight = zlight[light];
 
-        pl->top[pl->maxx + 1] = 0xffff;
-        pl->top[pl->minx - 1] = 0xffff;
+        pl->top[pl->maxx + 1] = 0xffffffff; // [crispy] hires / 32-bit integer math
+        pl->top[pl->minx - 1] = 0xffffffff; // [crispy] hires / 32-bit integer math
 
         stop = pl->maxx + 1;
         for (x = pl->minx; x <= stop; x++)
