@@ -27,6 +27,8 @@
 #include "s_sound.h"
 #include "v_video.h"
 #include "i_swap.h"
+#include "am_map.h"
+
 
 // TYPES -------------------------------------------------------------------
 
@@ -83,10 +85,10 @@ static void CheatRevealFunc(player_t * player, Cheat_t * cheat);
 static void CheatTrackFunc1(player_t * player, Cheat_t * cheat);
 static void CheatTrackFunc2(player_t * player, Cheat_t * cheat);
 
-// EXTERNAL DATA DECLARATIONS ----------------------------------------------
+// [crispy] new cheat functions
+static void CheatShowFpsFunc(player_t * player, Cheat_t * cheat);
 
-extern int ArmorIncrement[NUMCLASSES][NUMARMOR];
-extern int AutoArmorSave[NUMCLASSES];
+// EXTERNAL DATA DECLARATIONS ----------------------------------------------
 
 // PUBLIC DATA DECLARATIONS ------------------------------------------------
 
@@ -215,6 +217,9 @@ cheatseq_t CheatTrackSeq1 = CHEAT("`", 0);
 
 cheatseq_t CheatTrackSeq2 = CHEAT("`", 2);
 
+// [crispy] new cheat sequences
+cheatseq_t CheatShowFpsSeq = CHEAT("showfps", 0); // Show FPS
+
 static Cheat_t Cheats[] = {
     {CheatTrackFunc1, &CheatTrackSeq1},
     {CheatTrackFunc2, &CheatTrackSeq2},
@@ -243,6 +248,9 @@ static Cheat_t Cheats[] = {
     {CheatScriptFunc2, &CheatScriptSeq2},
     {CheatScriptFunc3, &CheatScriptSeq3},
     {CheatRevealFunc, &CheatRevealSeq},
+
+    // [crispy] new cheats
+    {CheatShowFpsFunc, &CheatShowFpsSeq},
 };
 
 #define SET_CHEAT(cheat, seq) \
@@ -744,7 +752,6 @@ static int oldpieces = -1;
 static int oldweapon = -1;
 static int oldkeys = -1;
 
-extern boolean automapactive;
 
 // [crispy] Needed to support widescreen status bar.
 void SB_ForceRedraw(void)
@@ -794,6 +801,8 @@ static void RefreshBackground(void)
                    ORIGSBARHEIGHT, 0, ORIGHEIGHT - ORIGSBARHEIGHT);
 }
 
+extern int right_widget_w; // [crispy]
+
 void SB_Drawer(void)
 {
     // Sound info debug stuff
@@ -952,6 +961,7 @@ static void DrawAnimatedIcons(void)
     if (CPlayer->powers[pw_invulnerability])
     {
         spindefense_x = 260 + WIDESCREENDELTA; // [crispy]
+        spindefense_x -= right_widget_w; // [crispy]
 
         if (CPlayer->powers[pw_invulnerability] > BLINKTHRESHOLD
             || !(CPlayer->powers[pw_invulnerability] & 16))
@@ -969,6 +979,7 @@ static void DrawAnimatedIcons(void)
     if (CPlayer->powers[pw_minotaur])
     {
         spinminotaur_x = 300 + WIDESCREENDELTA; // [crispy]
+        spinminotaur_x -= right_widget_w; // [crispy]
 
         if (CPlayer->powers[pw_minotaur] > BLINKTHRESHOLD
             || !(CPlayer->powers[pw_minotaur] & 16))
@@ -1368,7 +1379,7 @@ void DrawKeyBar(void)
     if (oldkeys != CPlayer->keys)
     {
         xPosition = 46;
-        for (i = 0; i < NUMKEYS && xPosition <= 126; i++)
+        for (i = 0; i < NUM_KEY_TYPES && xPosition <= 126; i++)
         {
             if (CPlayer->keys & (1 << i))
             {
@@ -1735,7 +1746,6 @@ static void CheatNoClipFunc(player_t * player, Cheat_t * cheat)
 static void CheatWeaponsFunc(player_t * player, Cheat_t * cheat)
 {
     int i;
-    //extern boolean *WeaponInShareware;
 
     for (i = 0; i < NUMARMOR; i++)
     {
@@ -1871,8 +1881,6 @@ static void CheatWarpFunc(player_t * player, Cheat_t * cheat)
 
 static void CheatPigFunc(player_t * player, Cheat_t * cheat)
 {
-    extern boolean P_UndoPlayerMorph(player_t * player);
-
     if (player->morphTics)
     {
         P_UndoPlayerMorph(player);
@@ -2024,7 +2032,6 @@ static void CheatScriptFunc3(player_t * player, Cheat_t * cheat)
     }
 }
 
-extern int cheating;
 
 static void CheatRevealFunc(player_t * player, Cheat_t * cheat)
 {
@@ -2100,3 +2107,17 @@ static void CheatTrackFunc2(player_t * player, Cheat_t * cheat)
         P_SetMessage(player, buffer, true);
     }
 }
+
+// [crispy] "Cheat" to show FPS
+static void CheatShowFpsFunc(player_t* player, Cheat_t* cheat)
+{
+    player->cheats ^= CF_SHOWFPS;
+    if (player->cheats & CF_SHOWFPS)
+    {
+        P_SetMessage(player, TXT_SHOWFPSON, false);
+    }
+    else
+    {
+        P_SetMessage(player, TXT_SHOWFPSOFF, false);
+    }
+}
