@@ -109,6 +109,8 @@ float mouse_acceleration_y = 1.0;
 int mouse_threshold_y = 0;
 int mouse_y_invert = 0;
 
+int runcentering = 1; // [crispy]
+
 // Translates the SDL key to a value of the type found in doomkeys.h
 static int TranslateKey(SDL_Keysym *sym)
 {
@@ -326,6 +328,8 @@ static void UpdateMouseButtonState(unsigned int button, boolean on)
     // Note: button "0" is left, button "1" is right,
     // button "2" is middle for Doom.  This is different
     // to how SDL sees things.
+    // In the end: Left=0, Right=1, Middle=2,
+    // WheelUp=3 WheelDown=4, XButton1=5, XButton2=6
 
     switch (button)
     {
@@ -343,7 +347,9 @@ static void UpdateMouseButtonState(unsigned int button, boolean on)
 
         default:
             // SDL buttons are indexed from 1.
-            --button;
+            // And the wheel is mapped to button 3/4
+            // So we have to increment 2 and decrement 1 => inc 1.
+            button++;
             break;
     }
 
@@ -514,6 +520,12 @@ void I_ReadMouse(void)
     }
 }
 
+// [crispy]
+void I_BindStrifeInputVariables(void)
+{
+    M_BindIntVariable("runcentering",              &runcentering);
+}
+
 // Bind all variables controlling input options.
 void I_BindInputVariables(void)
 {
