@@ -25,8 +25,6 @@
 #include "SDL.h"
 #include "SDL_mixer.h"
 
-#include "i_winmusic.h"
-
 #include "config.h"
 #include "doomtype.h"
 #include "memio.h"
@@ -45,7 +43,6 @@
 #include "z_zone.h"
 
 
-char *fluidsynth_sf_path = "";
 char *timidity_cfg_path = "";
 
 static char *temp_timidity_cfg = NULL;
@@ -138,12 +135,6 @@ static boolean music_initialized = false;
 
 static boolean sdl_was_initialized = false;
 
-#if defined(_WIN32)
-static boolean win_midi_stream_opened = false;
-#endif
-
-static boolean win_midi_song_registered = false;
-
 static boolean musicpaused = false;
 static int current_music_volume;
 
@@ -165,13 +156,6 @@ static void I_SDL_ShutdownMusic(void)
 {
     if (music_initialized)
     {
-#if defined(_WIN32)
-        if (win_midi_stream_opened)
-        {
-            I_WIN_ShutdownMusic();
-            win_midi_stream_opened = false;
-        }
-#endif
         Mix_HaltMusic();
         music_initialized = false;
 
@@ -195,8 +179,6 @@ static boolean SDLIsInitialized(void)
 // Initialize music subsystem
 static boolean I_SDL_InitMusic(void)
 {
-    boolean fluidsynth_sf_is_set = false;
-
     // If SDL_mixer is not initialized, we have to initialize it
     // and have the responsibility to shut it down later on.
 
@@ -225,24 +207,6 @@ static boolean I_SDL_InitMusic(void)
         }
     }
 
-    // When using FluidSynth, proceed to set the soundfont path via
-    // Mix_SetSoundFonts if necessary. We need to do this before calling
-    // Mix_Init() in order for FluidSynth to be registered as a valid decoder
-    // in the Mix_GetMusicDecoder() list.
-
-    if ((strlen(fluidsynth_sf_path) > 0) && (strlen(timidity_cfg_path) == 0))
-    {
-        if (M_FileExists(fluidsynth_sf_path))
-        {
-            Mix_SetSoundFonts(fluidsynth_sf_path);
-        }
-        else
-        {
-            fprintf(stderr,
-                    "I_SDL_InitMusic: Can't find FluidSynth soundfont.\n");
-        }
-    }
-
     // Initialize SDL_Mixer for MIDI music playback
     Mix_Init(MIX_INIT_MID | MIX_INIT_FLAC | MIX_INIT_OGG | MIX_INIT_MP3); // [crispy] initialize some more audio formats
 
@@ -251,35 +215,6 @@ static boolean I_SDL_InitMusic(void)
 
     RemoveTimidityConfig();
 
-    // If a soundfont has been set (either here on in the environment),
-    // confirm that FluidSynth is actually available before trying to use it.
-    if ((Mix_GetSoundFonts() != NULL) && (strlen(timidity_cfg_path) == 0))
-    {
-        int total;
-
-        total = Mix_GetNumMusicDecoders();
-
-        // If FluidSynth is present and has a valid soundfont, it will be in
-        // the list of available music decoders.
-        for (int i = 0; i < total; ++i)
-        {
-            if (!strcmp(Mix_GetMusicDecoder(i), "FLUIDSYNTH"))
-            {
-                fluidsynth_sf_is_set = true;
-                break;
-            }
-        }
-
-        if (fluidsynth_sf_is_set)
-        {
-            printf("I_SDL_InitMusic: Using FluidSynth.\n");
-        }
-        else
-        {
-            fprintf(stderr, "I_SDL_InitMusic: FluidSynth unavailable.\n");
-        }
-    }
-
     // If snd_musiccmd is set, we need to call Mix_SetMusicCMD to
     // configure an external music playback program.
 
@@ -288,15 +223,6 @@ static boolean I_SDL_InitMusic(void)
         Mix_SetMusicCMD(snd_musiccmd);
     }
 
-#if defined(_WIN32)
-    // Don't enable it for GUS or Fluidsynth, since they handle their own volume
-    // just fine.
-    if (snd_musicdevice != SNDDEVICE_GUS && !fluidsynth_sf_is_set)
-    {
-        win_midi_stream_opened = I_WIN_InitMusic();
-    }
-#endif
-
     return music_initialized;
 }
 
@@ -318,9 +244,6 @@ static void UpdateMusicVolume(void)
         vol = (current_music_volume * MIX_MAX_VOLUME) / 127;
     }
 
-#if defined(_WIN32)
-    I_WIN_SetMusicVolume(vol);
-#endif
     Mix_VolumeMusic(vol);
 }
 
@@ -345,7 +268,7 @@ static void I_SDL_PlaySong(void *handle, boolean looping)
         return;
     }
 
-    if (handle == NULL && !win_midi_song_registered)
+    if (handle == NULL)
     {
         return;
     }
@@ -359,16 +282,7 @@ static void I_SDL_PlaySong(void *handle, boolean looping)
         loops = 1;
     }
 
-#if defined(_WIN32)
-    if (win_midi_song_registered)
-    {
-        I_WIN_PlaySong(looping);
-    }
-    else
-#endif
-    {
-        Mix_PlayMusic((Mix_Music *) handle, loops);
-    }
+    Mix_PlayMusic((Mix_Music *) handle, loops);
 }
 
 static void I_SDL_PauseSong(void)
@@ -378,18 +292,9 @@ static void I_SDL_PauseSong(void)
         return;
     }
 
-#if defined(_WIN32)
-    if (win_midi_stream_opened)
-    {
-        I_WIN_PauseSong();
-    }
-    else
-#endif
-    {
-        musicpaused = true;
+    musicpaused = true;
 
-        UpdateMusicVolume();
-    }
+    UpdateMusicVolume();
 }
 
 static void I_SDL_ResumeSong(void)
@@ -399,18 +304,9 @@ static void I_SDL_ResumeSong(void)
         return;
     }
 
-#if defined(_WIN32)
-    if (win_midi_stream_opened)
-    {
-        I_WIN_ResumeSong();
-    }
-    else
-#endif
-    {
-        musicpaused = false;
+    musicpaused = false;
 
-        UpdateMusicVolume();
-    }
+    UpdateMusicVolume();
 }
 
 static void I_SDL_StopSong(void)
@@ -420,16 +316,7 @@ static void I_SDL_StopSong(void)
         return;
     }
 
-#if defined(_WIN32)
-    if (win_midi_song_registered)
-    {
-        I_WIN_StopSong();
-    }
-    else
-#endif
-    {
-        Mix_HaltMusic();
-    }
+    Mix_HaltMusic();
 }
 
 static void I_SDL_UnRegisterSong(void *handle)
@@ -441,33 +328,10 @@ static void I_SDL_UnRegisterSong(void *handle)
         return;
     }
 
-#if defined(_WIN32)
-    if (win_midi_song_registered)
+    if (handle != NULL)
     {
-        I_WIN_UnRegisterSong();
-        win_midi_song_registered = false;
+        Mix_FreeMusic(music);
     }
-    else
-#endif
-    {
-        if (handle != NULL)
-        {
-            Mix_FreeMusic(music);
-        }
-    }
-}
-
-// Determine whether memory block is a .mid file 
-#if defined(_WIN32)
-static boolean IsMid(byte *mem, int len)
-{
-    return len > 4 && !memcmp(mem, "MThd", 4);
-}
-#endif
-
-static boolean IsMus(byte *mem, int len)
-{
-    return len > 4 && !memcmp(mem, "MUS\x1a", 4);
 }
 
 static boolean ConvertMus(byte *musdata, int len, const char *filename)
@@ -532,41 +396,21 @@ static void *I_SDL_RegisterSong(void *data, int len)
     // by now, but Mix_SetMusicCMD() only works with Mix_LoadMUS(), so
     // we have to generate a temporary file.
 
-#if defined(_WIN32)
-    // If we do not have an external music command defined, play
-    // music with the Windows native MIDI.
-    if (win_midi_stream_opened && (IsMus(data, len) || IsMid(data, len)))
+    music = Mix_LoadMUS(filename);
+    if (music == NULL)
     {
-        if (I_WIN_RegisterSong(filename))
-        {
-            music = (void *) 1;
-            win_midi_song_registered = true;
-        }
-        else
-        {
-            music = NULL;
-            fprintf(stderr, "Error loading midi: Failed to register song.\n");
-        }
+        // Failed to load
+        fprintf(stderr, "Error loading midi: %s\n", Mix_GetError());
     }
-    else
-#endif
+
+    // Remove the temporary MIDI file; however, when using an external
+    // MIDI program we can't delete the file. Otherwise, the program
+    // won't find the file to play. This means we leave a mess on
+    // disk :(
+
+    if (strlen(snd_musiccmd) == 0)
     {
-        music = Mix_LoadMUS(filename);
-        if (music == NULL)
-        {
-            // Failed to load
-            fprintf(stderr, "Error loading midi: %s\n", Mix_GetError());
-        }
-
-        // Remove the temporary MIDI file; however, when using an external
-        // MIDI program we can't delete the file. Otherwise, the program
-        // won't find the file to play. This means we leave a mess on
-        // disk :(
-
-        if (strlen(snd_musiccmd) == 0)
-        {
-            M_remove(filename);
-        }
+        M_remove(filename);
     }
 
     free(filename);
@@ -585,7 +429,7 @@ static boolean I_SDL_MusicIsPlaying(void)
     return Mix_PlayingMusic();
 }
 
-static snddevice_t music_sdl_devices[] =
+static const snddevice_t music_sdl_devices[] =
 {
     SNDDEVICE_PAS,
     SNDDEVICE_GUS,
@@ -595,7 +439,7 @@ static snddevice_t music_sdl_devices[] =
     SNDDEVICE_AWE32,
 };
 
-music_module_t music_sdl_module =
+const music_module_t music_sdl_module =
 {
     music_sdl_devices,
     arrlen(music_sdl_devices),
