@@ -1010,11 +1010,97 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_STRING(music_pack_path),
 
+#ifdef HAVE_FLUIDSYNTH
+    //!
+    // If 1, activate the FluidSynth chorus effects module. If 0, no chorus
+    // will be added to the output signal.
+    //
+
+    CONFIG_VARIABLE_INT(fsynth_chorus_active),
+
+    //!
+    // Specifies the modulation depth of the FluidSynth chorus. Default is
+    // 5.0, range is 0.0 to 256.0.
+    //
+
+    CONFIG_VARIABLE_FLOAT(fsynth_chorus_depth),
+
+    //!
+    // Specifies the output amplitude of the FluidSynth chorus signal. Default
+    // is 0.35, range is 0.0 to 10.0.
+    //
+
+    CONFIG_VARIABLE_FLOAT(fsynth_chorus_level),
+
+    //!
+    // Sets the voice count of the FluidSynth chorus signal. Default is 3,
+    // range is 0 to 99.
+    //
+
+    CONFIG_VARIABLE_INT(fsynth_chorus_nr),
+
+    //!
+    // Sets the FluidSynth chorus modulation speed in Hz. Default is 0.3,
+    // range is 0.1 to 5.0.
+    //
+
+    CONFIG_VARIABLE_FLOAT(fsynth_chorus_speed),
+
+    //!
+    // This setting defines how FluidSynth interprets Bank Select messages. The
+    // default is "gs". Other possible values are "gm", "xg" and "mma".
+    //
+
+    CONFIG_VARIABLE_STRING(fsynth_midibankselect),
+
+    //!
+    // Sets the number of FluidSynth voices that can be played in parallel.
+    // Default is 256, range is 1 - 65535.
+    //
+
+    CONFIG_VARIABLE_INT(fsynth_polyphony),
+
+    //!
+    // If 1, activate the FluidSynth reverb effects module. If 0, no reverb
+    // will be added to the output signal.
+    //
+
+    CONFIG_VARIABLE_INT(fsynth_reverb_active),
+
+    //!
+    // Sets the amount of FluidSynth reverb damping. Default is 0.4, range is
+    // 0.0 to 1.0.
+    //
+
+    CONFIG_VARIABLE_FLOAT(fsynth_reverb_damp),
+
+    //!
+    // Sets the FluidSynth reverb amplitude. Default is 0.15, range is 0.0 -
+    // 1.0.
+    //
+
+    CONFIG_VARIABLE_FLOAT(fsynth_reverb_level),
+
+    //!
+    // Sets the room size(i.e. amount of wet) FluidSynth reverb. Default is
+    // 0.6, range is 0.0 - 1.0.
+    //
+
+    CONFIG_VARIABLE_FLOAT(fsynth_reverb_roomsize),
+
+    //!
+    // Sets the stereo spread of the FluidSynth reverb signal. Default is
+    // 0.4, range is 0.0 - 100.0.
+    //
+
+    CONFIG_VARIABLE_FLOAT(fsynth_reverb_width),
+
     //!
     // Full path to a soundfont file to use with FluidSynth MIDI playback.
     //
 
-    CONFIG_VARIABLE_STRING(fluidsynth_sf_path),
+    CONFIG_VARIABLE_STRING(fsynth_sf_path),
+#endif // HAVE_FLUIDSYNTH
 
     //!
     // Full path to a Timidity configuration file to use for MIDI
@@ -1039,6 +1125,39 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(gus_ram_kb),
 
+#ifdef _WIN32
+    //!
+    // MIDI device for native Windows MIDI.
+    //
+
+    CONFIG_VARIABLE_STRING(winmm_midi_device),
+
+    //!
+    // Reset device type for native Windows MIDI, default 1. Valid values are
+    // 0 (None), 1 (GS Mode), 2 (GM Mode), 3 (GM2 Mode), 4 (XG Mode).
+    //
+
+    CONFIG_VARIABLE_INT(winmm_reset_type),
+
+    //!
+    // Reset device delay for native Windows MIDI, default 0, median value 100 ms.
+    //
+
+    CONFIG_VARIABLE_INT(winmm_reset_delay),
+
+    //!
+    // Reverb level for native Windows MIDI, default -1, range 0-127.
+    //
+
+    CONFIG_VARIABLE_INT(winmm_reverb_level),
+
+    //!
+    // Chorus level for native Windows MIDI, default -1, range 0-127.
+    //
+
+    CONFIG_VARIABLE_INT(winmm_chorus_level),
+#endif
+
     //!
     // @game doom strife
     //
@@ -1069,6 +1188,15 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(vanilla_keyboard_mapping),
 
+    //!
+    // @game strife
+    //
+    // By default, pressing "run" centers the view in Strife. This behavior can
+    // be toggled with this setting.
+    //
+
+    CONFIG_VARIABLE_INT(runcentering),
+
     //!
     // If zero, this disables sectors changing their light level.
     //
@@ -1218,6 +1346,14 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(mouseb_useartifact),
 
+    //!
+    // @game strife
+    //
+    // Mouse button to use inventory item.
+    //
+
+    CONFIG_VARIABLE_INT(mouseb_invuse),
+
     //!
     // If non-zero, double-clicking a mouse button acts like pressing
     // the "use" key to use an object in-game, eg. a door or switch.
@@ -2095,6 +2231,14 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(crispy_crosshairtype),
 
+    //!
+    // @game doom heretic hexen
+    //
+    // Default difficulty when starting a new game.
+    //
+
+    CONFIG_VARIABLE_INT(crispy_defaultskill),
+
     //!
     // @game doom
     //
@@ -2135,6 +2279,14 @@ static default_t extra_defaults_list[] =
 
     CONFIG_VARIABLE_INT(crispy_flipcorpses),
 
+    //!
+    // @game doom heretic hexen
+    //
+    // Limit framerate to this value in frames per second.
+    //
+
+    CONFIG_VARIABLE_INT(crispy_fpslimit),
+
     //!
     // @game doom
     //
