@@ -62,11 +62,13 @@ void BindCompatibilityVariables(void)
         M_BindIntVariable("crispy_crosshairhealth", &crispy->crosshairhealth);
         M_BindIntVariable("crispy_crosshairtarget", &crispy->crosshairtarget);
         M_BindIntVariable("crispy_crosshairtype",   &crispy->crosshairtype);
+        M_BindIntVariable("crispy_defaultskill",    &crispy->defaultskill);
         M_BindIntVariable("crispy_demobar",         &crispy->demobar);
         M_BindIntVariable("crispy_demotimer",       &crispy->demotimer);
         M_BindIntVariable("crispy_demotimerdir",    &crispy->demotimerdir);
         M_BindIntVariable("crispy_extautomap",      &crispy->extautomap);
         M_BindIntVariable("crispy_flipcorpses",     &crispy->flipcorpses);
+        M_BindIntVariable("crispy_fpslimit",        &crispy->fpslimit);
         M_BindIntVariable("crispy_freeaim",         &crispy->freeaim);
         M_BindIntVariable("crispy_freelook",        &crispy->freelook);
         M_BindIntVariable("crispy_hires",           &crispy->hires);
@@ -98,6 +100,10 @@ void BindCompatibilityVariables(void)
         M_BindIntVariable("vanilla_savegame_limit", &vanilla_savegame_limit);
         M_BindIntVariable("vanilla_demo_limit",     &vanilla_demo_limit);
         M_BindIntVariable("crispy_automapstats",    &crispy->automapstats);
+        M_BindIntVariable("crispy_bobfactor",       &crispy->bobfactor);
+        M_BindIntVariable("crispy_centerweapon",    &crispy->centerweapon);
+        M_BindIntVariable("crispy_defaultskill",    &crispy->defaultskill);
+        M_BindIntVariable("crispy_fpslimit",        &crispy->fpslimit);
         M_BindIntVariable("crispy_freelook",        &crispy->freelook_hh);
         M_BindIntVariable("crispy_hires",           &crispy->hires);
         M_BindIntVariable("crispy_brightmaps",      &crispy->brightmaps);
@@ -105,6 +111,7 @@ void BindCompatibilityVariables(void)
         M_BindIntVariable("crispy_playercoords",    &crispy->playercoords);
         M_BindIntVariable("crispy_secretmessage",   &crispy->secretmessage);
         M_BindIntVariable("crispy_smoothscaling",   &crispy->smoothscaling);
+        M_BindIntVariable("crispy_soundmono",       &crispy->soundmono);
         M_BindIntVariable("crispy_uncapped",        &crispy->uncapped);
         M_BindIntVariable("crispy_vsync",           &crispy->vsync);
         M_BindIntVariable("crispy_widescreen",      &crispy->widescreen);
@@ -113,14 +120,47 @@ void BindCompatibilityVariables(void)
     {
         M_BindIntVariable("vanilla_savegame_limit", &vanilla_savegame_limit);
         M_BindIntVariable("vanilla_demo_limit",     &vanilla_demo_limit);
+        M_BindIntVariable("crispy_bobfactor",       &crispy->bobfactor);
+        M_BindIntVariable("crispy_centerweapon",    &crispy->centerweapon);
+        M_BindIntVariable("crispy_defaultskill",    &crispy->defaultskill);
+        M_BindIntVariable("crispy_fpslimit",        &crispy->fpslimit);
         M_BindIntVariable("crispy_freelook",        &crispy->freelook_hh);
         M_BindIntVariable("crispy_hires",           &crispy->hires);
+        M_BindIntVariable("crispy_playercoords",    &crispy->playercoords);
         M_BindIntVariable("crispy_smoothscaling",   &crispy->smoothscaling);
+        M_BindIntVariable("crispy_soundmono",       &crispy->soundmono);
         M_BindIntVariable("crispy_uncapped",        &crispy->uncapped);
         M_BindIntVariable("crispy_vsync",           &crispy->vsync);
         M_BindIntVariable("crispy_widescreen",      &crispy->widescreen);
         M_BindIntVariable("crispy_brightmaps",      &crispy->brightmaps);
     }
+    else if (gamemission == strife)
+    {
+        M_BindIntVariable("vanilla_savegame_limit", &vanilla_savegame_limit);
+        M_BindIntVariable("vanilla_demo_limit",     &vanilla_demo_limit);
+        M_BindIntVariable("crispy_automapoverlay",  &crispy->automapoverlay);
+        M_BindIntVariable("crispy_automaprotate",   &crispy->automaprotate);
+        M_BindIntVariable("crispy_bobfactor",       &crispy->bobfactor);
+        M_BindIntVariable("crispy_centerweapon",    &crispy->centerweapon);
+        M_BindIntVariable("crispy_crosshair",       &crispy->crosshair);
+        M_BindIntVariable("crispy_crosshairhealth", &crispy->crosshairhealth);
+        M_BindIntVariable("crispy_defaultskill",    &crispy->defaultskill);
+        M_BindIntVariable("crispy_fpslimit",        &crispy->fpslimit);
+        M_BindIntVariable("crispy_freelook",        &crispy->freelook_hh);
+        M_BindIntVariable("crispy_hires",           &crispy->hires);
+        M_BindIntVariable("crispy_leveltime",       &crispy->leveltime);
+        M_BindIntVariable("crispy_mouselook",       &crispy->mouselook);
+        M_BindIntVariable("crispy_playercoords",    &crispy->playercoords);
+        M_BindIntVariable("crispy_smoothlight",     &crispy->smoothlight);
+        M_BindIntVariable("crispy_smoothmap",       &crispy->smoothmap);
+        M_BindIntVariable("crispy_smoothscaling",   &crispy->smoothscaling);
+        M_BindIntVariable("crispy_soundfix",        &crispy->soundfix);
+        M_BindIntVariable("crispy_soundfull",       &crispy->soundfull);
+        M_BindIntVariable("crispy_soundmono",       &crispy->soundmono);
+        M_BindIntVariable("crispy_uncapped",        &crispy->uncapped);
+        M_BindIntVariable("crispy_vsync",           &crispy->vsync);
+        M_BindIntVariable("crispy_widescreen",      &crispy->widescreen);
+    }
     else
     {
     M_BindIntVariable("vanilla_savegame_limit", &vanilla_savegame_limit);
