@@ -14,6 +14,12 @@
 
 // Sound control menu
 
+#ifdef _WIN32
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
+#include <mmsystem.h>
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 
@@ -43,7 +49,10 @@ static const char *opltype_strings[] =
 };
 
 static const char *cfg_extension[] = { "cfg", NULL };
+
+#ifdef HAVE_FLUIDSYNTH
 static const char *sf_extension[] = { "sf2", "sf3", NULL };
+#endif
 
 // Config file variables:
 
@@ -61,18 +70,44 @@ static int numChannels = 8;
 static int sfxVolume = 8;
 static int musicVolume = 8;
 static int voiceVolume = 15;
-static int show_talk = 0;
+static int show_talk = 1; // [crispy] show subtitles by default
 // [crispy] values 3 and higher might reproduce DOOM.EXE more accurately,
 // but 1 is closer to "use_libsamplerate = 0" which is the default in Choco
 // and causes only a short delay at startup
-static int use_libsamplerate = 1;
-static float libsamplerate_scale = 0.65;
+int use_libsamplerate = 1;
+float libsamplerate_scale = 0.65;
 
-static char *music_pack_path = NULL;
-static char *timidity_cfg_path = NULL;
-static char *fluidsynth_sf_path = NULL;
+char *music_pack_path = NULL;
+char *timidity_cfg_path = NULL;
 static char *gus_patch_path = NULL;
 static int gus_ram_kb = 1024;
+#ifdef _WIN32
+#define MAX_MIDI_DEVICES 20
+static char **midi_names;
+static int midi_num_devices;
+static int midi_index;
+char *winmm_midi_device = NULL;
+int winmm_reset_type = -1;
+int winmm_reset_delay = 0;
+int winmm_reverb_level = -1;
+int winmm_chorus_level = -1;
+#endif
+
+#ifdef HAVE_FLUIDSYNTH
+char *fsynth_sf_path = NULL;
+int fsynth_chorus_active = 1;
+float fsynth_chorus_depth = 5.0f;
+float fsynth_chorus_level = 0.35f;
+int fsynth_chorus_nr = 3;
+float fsynth_chorus_speed = 0.3f;
+char *fsynth_midibankselect = "gs";
+int fsynth_polyphony = 256;
+int fsynth_reverb_active = 1;
+float fsynth_reverb_damp = 0.4f;
+float fsynth_reverb_level = 0.15f;
+float fsynth_reverb_roomsize = 0.6f;
+float fsynth_reverb_width = 4.0f;
+#endif // HAVE_FLUIDSYNTH
 
 // DOS specific variables: these are unused but should be maintained
 // so that the config file can be shared between chocolate
@@ -128,6 +163,90 @@ static void OpenMusicPackDir(TXT_UNCAST_ARG(widget), TXT_UNCAST_ARG(unused))
     }
 }
 
+#ifdef _WIN32
+static void UpdateMidiDevice(TXT_UNCAST_ARG(widget), TXT_UNCAST_ARG(data))
+{
+    free(winmm_midi_device);
+    winmm_midi_device = M_StringDuplicate(midi_names[midi_index]);
+}
+
+static txt_dropdown_list_t *MidiDeviceSelector(void)
+{
+    txt_dropdown_list_t *result;
+    int all_devices;
+    int device_ids[MAX_MIDI_DEVICES];
+    MMRESULT mmr;
+    MIDIOUTCAPS mcaps;
+    int i;
+
+    if (midi_num_devices > 0)
+    {
+        for (i = 0; i < midi_num_devices; ++i)
+        {
+            free(midi_names[i]);
+            midi_names[i] = NULL;
+        }
+        free(midi_names);
+        midi_names = NULL;
+    }
+    midi_num_devices = 0;
+
+    // get the number of midi devices on this system
+    all_devices = midiOutGetNumDevs() + 1; // include MIDI_MAPPER
+    if (all_devices > MAX_MIDI_DEVICES)
+    {
+        all_devices = MAX_MIDI_DEVICES;
+    }
+
+    // get the valid device ids only, starting from -1 (MIDI_MAPPER)
+    for (i = 0; i < all_devices; ++i)
+    {
+        mmr = midiOutGetDevCaps(i - 1, &mcaps, sizeof(mcaps));
+        if (mmr == MMSYSERR_NOERROR)
+        {
+            device_ids[midi_num_devices] = i - 1;
+            midi_num_devices++;
+        }
+    }
+
+    // get the device names
+    midi_names = malloc(midi_num_devices * sizeof(char *));
+    for (i = 0; i < midi_num_devices; ++i)
+    {
+        mmr = midiOutGetDevCaps(device_ids[i], &mcaps, sizeof(mcaps));
+        if (mmr == MMSYSERR_NOERROR)
+        {
+            midi_names[i] = M_StringDuplicate(mcaps.szPname);
+        }
+    }
+
+    // set the dropdown list index to the previously selected device
+    for (i = 0; i < midi_num_devices; ++i)
+    {
+        if (winmm_midi_device != NULL &&
+            strstr(winmm_midi_device, midi_names[i]))
+        {
+            midi_index = i;
+            break;
+        }
+        else if (winmm_midi_device == NULL || i == midi_num_devices - 1)
+        {
+            // give up and use MIDI_MAPPER
+            midi_index = 0;
+            free(winmm_midi_device);
+            winmm_midi_device = M_StringDuplicate(midi_names[0]);
+            break;
+        }
+    }
+
+    result = TXT_NewDropdownList(&midi_index, (const char **)midi_names,
+                                 midi_num_devices);
+    TXT_SignalConnect(result, "changed", UpdateMidiDevice, NULL);
+
+    return result;
+}
+#endif
+
 void ConfigSound(TXT_UNCAST_ARG(widget), void *user_data)
 {
     txt_window_t *window;
@@ -192,7 +311,15 @@ void ConfigSound(TXT_UNCAST_ARG(widget), void *user_data)
                                     TXT_DIRECTORY),
                 NULL)),
 
-        TXT_NewRadioButton("MIDI/MP3/OGG/FLAC", &snd_musicdevice, SNDDEVICE_GENMIDI), // [crispy] improve ambigious music backend name
+        TXT_NewRadioButton("Native MIDI", &snd_musicdevice, SNDDEVICE_GENMIDI),
+#ifdef _WIN32
+        TXT_NewConditional(&snd_musicdevice, SNDDEVICE_GENMIDI,
+            TXT_NewHorizBox(
+                TXT_NewStrut(4, 0),
+                TXT_NewLabel("Device: "),
+                MidiDeviceSelector(),
+                NULL)),
+#endif
         TXT_NewConditional(&snd_musicdevice, SNDDEVICE_GENMIDI,
             TXT_MakeTable(2,
                 TXT_NewStrut(4, 0),
@@ -201,13 +328,19 @@ void ConfigSound(TXT_UNCAST_ARG(widget), void *user_data)
                 TXT_NewFileSelector(&timidity_cfg_path, 34,
                                     "Select Timidity config file",
                                     cfg_extension),
+                NULL)),
+#ifdef HAVE_FLUIDSYNTH
+        TXT_NewRadioButton("FluidSynth", &snd_musicdevice, SNDDEVICE_FSYNTH),
+        TXT_NewConditional(&snd_musicdevice, SNDDEVICE_FSYNTH,
+            TXT_MakeTable(2,
                 TXT_NewStrut(4, 0),
-                TXT_NewLabel("FluidSynth soundfont file: "),
+                TXT_NewLabel("Soundfont file: "),
                 TXT_NewStrut(4, 0),
-                TXT_NewFileSelector(&fluidsynth_sf_path, 34,
+                TXT_NewFileSelector(&fsynth_sf_path, 34,
                                     "Select FluidSynth soundfont file",
                                     sf_extension),
                 NULL)),
+#endif
         NULL);
 }
 
@@ -227,7 +360,29 @@ void BindSoundVariables(void)
     M_BindStringVariable("gus_patch_path",        &gus_patch_path);
     M_BindStringVariable("music_pack_path",     &music_pack_path);
     M_BindStringVariable("timidity_cfg_path",     &timidity_cfg_path);
-    M_BindStringVariable("fluidsynth_sf_path",    &fluidsynth_sf_path);
+#ifdef _WIN32
+    M_BindStringVariable("winmm_midi_device",     &winmm_midi_device);
+    M_BindIntVariable("winmm_reset_type",         &winmm_reset_type);
+    M_BindIntVariable("winmm_reset_delay",        &winmm_reset_delay);
+    M_BindIntVariable("winmm_reverb_level",       &winmm_reverb_level);
+    M_BindIntVariable("winmm_chorus_level",       &winmm_chorus_level);
+#endif
+
+#ifdef HAVE_FLUIDSYNTH
+    M_BindIntVariable("fsynth_chorus_active",     &fsynth_chorus_active);
+    M_BindFloatVariable("fsynth_chorus_depth",    &fsynth_chorus_depth);
+    M_BindFloatVariable("fsynth_chorus_level",    &fsynth_chorus_level);
+    M_BindIntVariable("fsynth_chorus_nr",         &fsynth_chorus_nr);
+    M_BindFloatVariable("fsynth_chorus_speed",    &fsynth_chorus_speed);
+    M_BindStringVariable("fsynth_midibankselect", &fsynth_midibankselect);
+    M_BindIntVariable("fsynth_polyphony",         &fsynth_polyphony);
+    M_BindIntVariable("fsynth_reverb_active",     &fsynth_reverb_active);
+    M_BindFloatVariable("fsynth_reverb_damp",     &fsynth_reverb_damp);
+    M_BindFloatVariable("fsynth_reverb_level",    &fsynth_reverb_level);
+    M_BindFloatVariable("fsynth_reverb_roomsize", &fsynth_reverb_roomsize);
+    M_BindFloatVariable("fsynth_reverb_width",    &fsynth_reverb_width);
+    M_BindStringVariable("fsynth_sf_path",        &fsynth_sf_path);
+#endif // HAVE_FLUIDSYNTH
 
     M_BindIntVariable("snd_sbport",               &snd_sbport);
     M_BindIntVariable("snd_sbirq",                &snd_sbirq);
@@ -251,7 +406,10 @@ void BindSoundVariables(void)
     music_pack_path = M_StringDuplicate("");
     timidity_cfg_path = M_StringDuplicate("");
     gus_patch_path = M_StringDuplicate("");
-    fluidsynth_sf_path = M_StringDuplicate("");
+
+#ifdef HAVE_FLUIDSYNTH
+    fsynth_sf_path = M_StringDuplicate("");
+#endif
 
     // All versions of Heretic and Hexen did pitch-shifting.
     // Most versions of Doom did not and Strife never did.
