@@ -114,12 +114,6 @@ boolean         randomparm;     // [STRIFE] haleyjd 20130915: checkparm of -rand
 boolean         showintro = true;   // [STRIFE] checkparm of -nograph, disables intro
 
 
-//extern int soundVolume;
-//extern  int	sfxVolume;
-//extern  int	musicVolume;
-
-extern  boolean	inhelpscreens;
-
 skill_t		startskill;
 int             startepisode;
 int		startmap;
@@ -163,6 +157,9 @@ static int comport = 0;
 // fraggle 06/03/11 [STRIFE]: Multiplayer nickname?
 char *nickname = NULL;
 
+// [crispy] track screen wipe
+boolean screenwipe;
+
 void D_ConnectNetGame(void);
 void D_CheckNetGame(void);
 
@@ -205,9 +202,6 @@ void D_ProcessEvents (void)
 // * 20110206: Start wipegamestate at GS_UNKNOWN (STRIFE-TODO: rename?)
 //
 gamestate_t     wipegamestate = GS_UNKNOWN;
-extern  boolean setsizeneeded;
-//extern  int             showMessages; [STRIFE] no such variable
-void R_ExecuteSetViewSize (void);
 
 void D_Display (void)
 {
@@ -242,6 +236,7 @@ void D_Display (void)
     // save the current screen if about to wipe
     if (gamestate != wipegamestate)
     {
+        screenwipe = true; // [crispy]
         wipe = true;
         wipe_StartScreen(0, 0, SCREENWIDTH, SCREENHEIGHT);
     }
@@ -257,15 +252,19 @@ void D_Display (void)
     case GS_LEVEL:
         if (!gametic)
             break;
-        if (automapactive)
+        if (automapactive && !crispy->automapoverlay)
+        {
+            // [crispy] update automap while playing
+            R_RenderPlayerView (&players[displayplayer]);
             AM_Drawer ();
-        if (wipe || (viewheight != (200 <<crispy->hires) && fullscreen) )
+        }
+        if (wipe || (viewheight != SCREENHEIGHT && fullscreen) )
             redrawsbar = true;
         // haleyjd 08/29/10: [STRIFE] Always redraw sbar if menu is/was active
         if (menuactivestate || (inhelpscreensstate && !inhelpscreens))
             redrawsbar = true;              // just put away the help screen
-        ST_Drawer (viewheight == (200 << crispy->hires), redrawsbar );
-        fullscreen = viewheight == (200 << crispy->hires);
+        ST_Drawer (viewheight == SCREENHEIGHT, redrawsbar );
+        fullscreen = viewheight == SCREENHEIGHT;
         break;
       
      // haleyjd 08/23/2010: [STRIFE] No intermission
@@ -291,7 +290,7 @@ void D_Display (void)
     I_UpdateNoBlit ();
 
     // draw the view directly
-    if (gamestate == GS_LEVEL && !automapactive && gametic)
+    if (gamestate == GS_LEVEL && (!automapactive || crispy->automapoverlay) && gametic)
         R_RenderPlayerView (&players[displayplayer]);
 
     // clean up border stuff
@@ -306,7 +305,7 @@ void D_Display (void)
     }
 
     // see if the border needs to be updated to the screen
-    if (gamestate == GS_LEVEL && !automapactive && scaledviewwidth != (320 << crispy->hires))
+    if (gamestate == GS_LEVEL && (!automapactive || crispy->automapoverlay) && scaledviewwidth != SCREENWIDTH)
     {
         if (menuactive || menuactivestate || !viewactivestate)
         {
@@ -336,6 +335,17 @@ void D_Display (void)
     // haleyjd 20120208: [STRIFE] Rogue moved this down to below border drawing
     if (gamestate == GS_LEVEL && gametic)
     {
+        // [crispy] in automap overlay mode, draw the automap and HUD on top of
+        // everything else (except for Strife popups and exit screen)
+        if (automapactive && crispy->automapoverlay)
+        {
+            AM_Drawer ();
+
+            // [crispy] force redraw of status bar and border
+            viewactivestate = false;
+            inhelpscreensstate = true;
+        }
+
         HU_Drawer ();
         if(ST_DrawExternal()) 
             popupactivestate = true;
@@ -346,14 +356,26 @@ void D_Display (void)
         }
     }
 
+    // [crispy] draw neither pause pic nor menu when taking a clean screenshot
+    if (crispy->cleanscreenshot && !wipe)
+    {
+        I_FinishUpdate();
+        if (crispy->post_rendering_hook)
+        {
+            crispy->post_rendering_hook();
+            crispy->post_rendering_hook = NULL;
+        }
+        return;
+    }
+
     // draw pause pic
     if (paused)
     {
-        if (automapactive)
+        if (automapactive && !crispy->automapoverlay)
             y = 4;
         else
             y = (viewwindowy >> crispy->hires)+4;
-        V_DrawPatchDirect((viewwindowx >> crispy->hires) + ((scaledviewwidth >> crispy->hires) - 68) / 2, y,
+        V_DrawPatchDirect((viewwindowx >> crispy->hires) + ((scaledviewwidth >> crispy->hires) - 68) / 2 - WIDESCREENDELTA, y,
                           W_CacheLumpName (DEH_String("M_PAUSE"), PU_CACHE));
     }
 
@@ -367,6 +389,15 @@ void D_Display (void)
     if (!wipe)
     {
         I_FinishUpdate ();              // page flip or blit buffer
+
+        // [crispy] post-rendering function pointer to apply config changes
+        // that affect rendering and that are better applied after the current
+        // frame has finished rendering
+        if (crispy->post_rendering_hook)
+        {
+            crispy->post_rendering_hook();
+            crispy->post_rendering_hook = NULL;
+        }
         return;
     }
     
@@ -392,6 +423,8 @@ void D_Display (void)
         M_Drawer ();                            // menu is drawn even on top of wipes
         I_FinishUpdate ();                      // page flip or blit buffer
     } while (!done);
+
+    screenwipe = false; // [crispy]
 }
 
 //
@@ -420,6 +453,7 @@ void D_BindVariables(void)
 
     M_ApplyPlatformDefaults();
 
+    I_BindStrifeInputVariables(); // [crispy]
     I_BindInputVariables();
     I_BindVideoVariables();
     I_BindJoystickVariables();
@@ -455,6 +489,8 @@ void D_BindVariables(void)
     // * Added nickname, comport
 
     M_BindIntVariable("mouse_sensitivity",      &mouseSensitivity);
+    M_BindIntVariable("mouse_sensitivity_x2",   &mouseSensitivity_x2); // [crispy]
+    M_BindIntVariable("mouse_sensitivity_y",    &mouseSensitivity_y); // [crispy]
     M_BindIntVariable("sfx_volume",             &sfxVolume);
     M_BindIntVariable("music_volume",           &musicVolume);
     M_BindIntVariable("voice_volume",           &voiceVolume); 
@@ -482,6 +518,30 @@ void D_BindVariables(void)
         M_snprintf(buf, sizeof(buf), "chatmacro%i", i);
         M_BindStringVariable(buf, &chat_macros[i]);
     }
+
+    // [crispy] bind "crispness" config variables
+    M_BindIntVariable("crispy_automapoverlay",  &crispy->automapoverlay);
+    M_BindIntVariable("crispy_automaprotate",   &crispy->automaprotate);
+    M_BindIntVariable("crispy_bobfactor",       &crispy->bobfactor);
+    M_BindIntVariable("crispy_centerweapon",    &crispy->centerweapon);
+    M_BindIntVariable("crispy_crosshair",       &crispy->crosshair);
+    M_BindIntVariable("crispy_crosshairhealth", &crispy->crosshairhealth);
+    M_BindIntVariable("crispy_defaultskill",    &crispy->defaultskill);
+    M_BindIntVariable("crispy_fpslimit",        &crispy->fpslimit);
+    M_BindIntVariable("crispy_freelook",        &crispy->freelook_hh);
+    M_BindIntVariable("crispy_hires",           &crispy->hires);
+    M_BindIntVariable("crispy_leveltime",       &crispy->leveltime);
+    M_BindIntVariable("crispy_mouselook",       &crispy->mouselook);
+    M_BindIntVariable("crispy_playercoords",    &crispy->playercoords);
+    M_BindIntVariable("crispy_smoothlight",     &crispy->smoothlight);
+    M_BindIntVariable("crispy_smoothmap",       &crispy->smoothmap);
+    M_BindIntVariable("crispy_smoothscaling",   &crispy->smoothscaling);
+    M_BindIntVariable("crispy_soundfix",        &crispy->soundfix);
+    M_BindIntVariable("crispy_soundfull",       &crispy->soundfull);
+    M_BindIntVariable("crispy_soundmono",       &crispy->soundmono);
+    M_BindIntVariable("crispy_uncapped",        &crispy->uncapped);
+    M_BindIntVariable("crispy_vsync",           &crispy->vsync);
+    M_BindIntVariable("crispy_widescreen",      &crispy->widescreen);
 }
 
 //
@@ -514,6 +574,15 @@ static boolean D_StartupGrabCallback(void)
     return false;
 }
 
+// [crispy]
+void EnableLoadingDisk(void)
+{
+    if (show_diskicon)
+    {
+        V_EnableLoadingDisk("STDISK", SCREENWIDTH - LOADING_DISK_W, 3);
+    }
+}
+
 //
 //  D_DoomLoop
 //
@@ -534,10 +603,7 @@ void D_DoomLoop (void)
         I_InitGraphics();
     }
 
-    if (show_diskicon)
-    {
-        V_EnableLoadingDisk("STDISK", SCREENWIDTH - LOADING_DISK_W, 3);
-    }
+    EnableLoadingDisk(); // [crispy]
     I_SetGrabMouseCallback(D_GrabMouseCallback);
 
     V_RestoreBuffer();
@@ -597,7 +663,7 @@ void D_PageTicker (void)
 //
 void D_PageDrawer (void)
 {
-    V_DrawPatch (0, 0, W_CacheLumpName(pagename, PU_CACHE));
+    V_DrawPatchFullScreen (W_CacheLumpName(pagename, PU_CACHE), false);
 }
 
 
@@ -627,6 +693,9 @@ void D_DoAdvanceDemo (void)
     paused = false;
     gameaction = ga_nothing;
     
+    // [crispy] update the "singleplayer" variable
+    CheckCrispySingleplayer(!demorecording && !demoplayback && !netgame);
+
     // villsa 09/12/10: [STRIFE] converted pagetics to ticrate
     switch (demosequence)
     {
@@ -754,6 +823,7 @@ void D_StartTitle (void)
 {
     gamestate = GS_DEMOSCREEN;
     gameaction = ga_nothing;
+    automapactive = false; // [crispy]
     demosequence = -2;
     D_AdvanceDemo ();
 }
@@ -1217,8 +1287,8 @@ boolean D_PatchClipCallback(patch_t *patch, int x, int y)
 {
     // note that offsets were already accounted for in V_DrawPatch
     return (x >= 0 && y >= 0 
-            && x + SHORT(patch->width) <= ORIGWIDTH 
-            && y + SHORT(patch->height) <= ORIGHEIGHT);
+            && x + SHORT(patch->width) <= (SCREENWIDTH >> crispy->hires)
+            && y + SHORT(patch->height) <= (SCREENHEIGHT >> crispy->hires));
 }
 
 //
@@ -1275,6 +1345,7 @@ static void D_IntroBackground(void)
 
     // Draw a 95-pixel rect from STARTUP0 starting at y=57 to (0,41) on the
     // screen (this was a memcpy directly to 0xA3340 in low DOS memory)
+    // [crispy] use scaled function
     V_DrawScaledBlock(0, 41, 320, 95, rawgfx_startup0 + (320*57));
 }
 
@@ -1405,7 +1476,8 @@ static void D_DrawIntroSequence(void)
         // Draw the laser
         // Blitted 16 bytes for 16 rows starting at 705280 + laserpos
         // (705280 - 0xA0000) / 320 == 156
-        V_DrawBlock(laserpos, 156, 16, 16, rawgfx_startlz[laserpos % 2]);
+        // [crispy] use scaled function
+        V_DrawScaledBlock(laserpos, 156, 16, 16, rawgfx_startlz[laserpos % 2]);
 
         // Robot position
         robotpos = laserpos % 5 - 2;
@@ -1413,12 +1485,14 @@ static void D_DrawIntroSequence(void)
         // Draw the robot
         // Blitted 48 bytes for 48 rows starting at 699534 + (320*robotpos)
         // 699534 - 0xA0000 == 44174, which % 320 == 14, / 320 == 138
+        // [crispy] use scaled function
         V_DrawScaledBlock(14, 138 + robotpos, 48, 48, rawgfx_startbot);
 
         // Draw the peasant
         // Blitted 32 bytes for 64 rows starting at 699142
         // 699142 - 0xA0000 == 43782, which % 320 == 262, / 320 == 136
-        V_DrawBlock(262, 136, 32, 64, rawgfx_startp[laserpos % 4]);
+        // [crispy] use scaled function
+        V_DrawScaledBlock(262, 136, 32, 64, rawgfx_startp[laserpos % 4]);
 
         I_FinishUpdate();
     }
@@ -1486,6 +1560,8 @@ static void G_CheckDemoStatusAtExit (void)
     G_CheckDemoStatus();
 }
 
+static const char *const loadparms[] = {"-file", "-merge", NULL}; // [crispy]
+
 //
 // D_DoomMain
 //
@@ -1718,8 +1794,6 @@ void D_DoomMain (void)
     if ( (p=M_CheckParm ("-turbo")) )
     {
         int     scale = 200;
-        extern int forwardmove[2];
-        extern int sidemove[2];
 
         if (p<myargc-1)
             scale = atoi (myargv[p+1]);
@@ -1745,6 +1819,9 @@ void D_DoomMain (void)
     D_BindVariables();
     M_LoadDefaults();
 
+    // [crispy] show exit screen only if showing ENDOOM
+    show_exitscreen = show_endoom;
+
     // Save configuration at exit.
     I_AtExit(M_SaveDefaults, false);
 
@@ -1807,6 +1884,31 @@ void D_DoomMain (void)
     // Debug:
 //    W_PrintDirectory();
 
+    // [crispy] add wad files from autoload PWAD directories
+
+    if (!M_ParmExists("-noautoload") && gamemode != shareware)
+    {
+        int i;
+
+        for (i = 0; loadparms[i]; i++)
+        {
+            int p;
+            p = M_CheckParmWithArgs(loadparms[i], 1);
+            if (p)
+            {
+                while (++p != myargc && myargv[p][0] != '-')
+                {
+                    char *autoload_dir;
+                    if ((autoload_dir = M_GetAutoloadDir(M_BaseName(myargv[p]), false)))
+                    {
+                        W_AutoLoadWADs(autoload_dir);
+                        free(autoload_dir);
+                    }
+                }
+            }
+        }
+    }
+
     //!
     // @arg <demo>
     // @category demo
@@ -1872,6 +1974,31 @@ void D_DoomMain (void)
 
     W_GenerateHashTable();
 
+    // [crispy] process .deh files from PWADs autoload directories
+
+    if (!M_ParmExists("-noautoload") && gamemode != shareware)
+    {
+        int i;
+
+        for (i = 0; loadparms[i]; i++)
+        {
+            int p;
+            p = M_CheckParmWithArgs(loadparms[i], 1);
+            if (p)
+            {
+                while (++p != myargc && myargv[p][0] != '-')
+                {
+                    char *autoload_dir;
+                    if ((autoload_dir = M_GetAutoloadDir(M_BaseName(myargv[p]), false)))
+                    {
+                        DEH_AutoLoadPatches(autoload_dir);
+                        free(autoload_dir);
+                    }
+                }
+            }
+        }
+    }
+
     InitGameVersion();
     InitTitleString();
     D_SetGameDescription();
@@ -1933,7 +2060,10 @@ void D_DoomMain (void)
     D_IntroTick(); // [STRIFE]
     
     // get skill / episode / map from parms
-    startskill = sk_easy; // [STRIFE]: inits to sk_easy
+
+    // [crispy] set defaultskill (Strife default is SKILL_HNTR)
+    startskill = (crispy->defaultskill + SKILL_HNTR) % NUM_SKILLS;
+
     startepisode = 1;
     startmap = 1;
     autostart = false;
@@ -2066,6 +2196,8 @@ void D_DoomMain (void)
         startloadgame = -1;
     }
 
+// [crispy] disable meaningless warning, we always use "-merge" anyway
+#if 0
     if (W_CheckNumForName("SS_START") >= 0
      || W_CheckNumForName("FF_END") >= 0)
     {
@@ -2074,6 +2206,7 @@ void D_DoomMain (void)
                " floor textures.  You may want to use the '-merge' command\n"
                " line option instead of '-file'.\n");
     }
+#endif
 
     I_PrintStartupBanner(gamedescription);
     PrintDehackedBanners();
