@@ -33,6 +33,7 @@
 #include "r_sky.h"
 #include "r_data.h"
 #include "sounds.h" // villsa [STRIFE]
+#include "v_trans.h" // [crispy] color translation and color string tables
 
 //
 // Graphics.
@@ -243,8 +244,6 @@ void R_GenerateComposite (int texnum)
     colofs = texturecolumnofs[texnum];
     
     // Composite the columns together.
-    patch = texture->patches;
-		
     for (i=0 , patch = texture->patches;
 	 i<texture->patchcount;
 	 i++, patch++)
@@ -315,7 +314,6 @@ void R_GenerateLookup (int texnum)
     //  with only a single patch are all done.
     patchcount = (byte *) Z_Malloc(texture->width, PU_STATIC, &patchcount);
     memset (patchcount, 0, texture->width);
-    patch = texture->patches;
 
     for (i=0 , patch = texture->patches;
 	 i<texture->patchcount;
@@ -464,7 +462,6 @@ void R_InitTextures (void)
     
     int*		patchlookup;
     
-    int			totalwidth;
     int			nummappatches;
     int			offset;
     int			maxoff;
@@ -522,8 +519,6 @@ void R_InitTextures (void)
     texturewidthmask = Z_Malloc (numtextures * sizeof(*texturewidthmask), PU_STATIC, 0);
     textureheight = Z_Malloc (numtextures * sizeof(*textureheight), PU_STATIC, 0);
 
-    totalwidth = 0;
-    
     //	Really complex printing shit...
     temp1 = W_GetNumForName (DEH_String("S_START"));  // P_???????
     temp2 = W_GetNumForName (DEH_String("S_END")) - 1;
@@ -603,8 +598,6 @@ void R_InitTextures (void)
 
         texturewidthmask[i] = j-1;
         textureheight[i] = texture->height<<FRACBITS;
-
-        totalwidth += texture->width;
     }
 
     Z_Free(patchlookup);
@@ -702,6 +695,30 @@ void R_InitColormaps (void)
     // Load in the light tables, 256 byte align tables.
     lump = W_GetNumForName(DEH_String("COLORMAP"));
     colormaps = W_CacheLumpNum(lump, PU_STATIC);
+
+    // [crispy] initialize color translation and color strings tables
+    {
+        byte *playpal = W_CacheLumpName("PLAYPAL", PU_STATIC);
+        char c[3];
+        int i, j;
+
+        if (!crstr)
+            crstr = I_Realloc(NULL, CRMAX * sizeof(*crstr));
+
+        // [crispy] CRMAX - 2: don't override the original GREN and BLUE2 Boom tables
+        for (i = 0; i < CRMAX - 2; i++)
+        {
+            for (j = 0; j < 256; j++)
+            {
+                cr[i][j] = V_Colorize(playpal, i, j, false);
+            }
+
+            M_snprintf(c, sizeof(c), "%c%c", cr_esc, '0' + i);
+            crstr[i] = M_StringDuplicate(c);
+        }
+
+        W_ReleaseLumpName("PLAYPAL");
+    }
 }
 
 
