@@ -19,10 +19,12 @@
 #ifndef __V_DISKICON__
 #define __V_DISKICON__
 
+#include "crispy.h"
+
 // Dimensions of the flashing "loading" disk icon
 
-#define LOADING_DISK_W (16 << hires)
-#define LOADING_DISK_H (16 << hires)
+#define LOADING_DISK_W (16 << crispy->hires)
+#define LOADING_DISK_H (16 << crispy->hires)
 
 extern void V_EnableLoadingDisk(const char *lump_name, int xoffs, int yoffs);
 extern void V_BeginRead(size_t nbytes);
